@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRdpt.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRdpt.c,v 1.5 2008/12/20 05:02:46 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -649,14 +649,12 @@ calmaElementPath()
  * calmaElementText --
  *
  * Read labels.
- * CURRENTLY UNIMPLEMENTED.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Currently, none.
- *	Eventually, may add labels to our label list.
+ *	Add labels to our label list.
  *
  * ----------------------------------------------------------------------------
  */
@@ -664,13 +662,15 @@ calmaElementPath()
 void
 calmaElementText()
 {
-    static int ignore[] = { CALMA_PRESENTATION, CALMA_PATHTYPE, CALMA_WIDTH,
-			    CALMA_STRANS, CALMA_MAG, CALMA_ANGLE, -1 };
+    static int ignore[] = { CALMA_PATHTYPE, CALMA_WIDTH, -1 };
     char *textbody = NULL;
     int nbytes, rtype;
     int layer, textt, cifnum;
     TileType type;
     Rect r;
+    unsigned short textpres;
+    double dval;
+    int size, angle, font, pos;
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -690,8 +690,87 @@ calmaElementText()
     }
     else type = cifCurReadStyle->crs_labelLayer[cifnum];
 
-    /* Skip presentation, pathtype, width, and transform */
-    calmaSkipSet(ignore);
+    font = -1;
+    angle = 0;
+
+    /* Default size is 1um */
+    size = (int)((800 * cifCurReadStyle->crs_multiplier)
+				/ cifCurReadStyle->crs_scaleFactor);
+    /* Default position is bottom-right (but what the spec calls "top-left"!) */
+    pos = GEO_SOUTHEAST;
+
+    /* Parse presentation and magnitude/angle part of transform */
+    /* Skip pathtype and width */
+
+    PEEKRH(nbytes, rtype);
+    if (nbytes > 0 && rtype == CALMA_PRESENTATION)
+    {
+	calmaReadI2Record(CALMA_PRESENTATION, &textpres);
+	font = (textpres >> 4) & 0x03;
+	switch (textpres & 0x000f)
+	{
+	    case 0x000a:
+		pos = GEO_NORTHWEST;
+		break;
+	    case 0x0006:
+		pos = GEO_WEST;
+		break;
+	    case 0x0002:
+		pos = GEO_SOUTHWEST;
+		break;
+	    case 0x0009:
+		pos = GEO_NORTH;
+		break;
+	    case 0x0005:
+		pos = GEO_CENTER;
+		break;
+	    case 0x0001:
+		pos = GEO_SOUTH;
+		break;
+	    case 0x0008:
+		pos = GEO_NORTHEAST;
+		break;
+	    case 0x0004:
+		pos = GEO_EAST;
+		break;
+	    case 0x0000:
+		pos = GEO_SOUTHEAST;
+		break;
+	}
+    }
+    else if (nbytes > 0 && rtype != CALMA_STRANS)
+	calmaSkipSet(ignore);
+
+    READRH(nbytes, rtype);
+    if (nbytes > 0 && rtype == CALMA_STRANS)
+    {
+	/* We don't handle the strans record for text */
+	calmaSkipBytes(nbytes - CALMAHEADERLENGTH);
+
+	READRH(nbytes, rtype);
+	if (nbytes > 0 && rtype == CALMA_MAG)
+	{
+	    calmaReadR8(&dval);
+	    /* Assume that MAG is the label size in microns	*/
+	    /* "size" is the label size in 8 * (database units) */
+	    size = (int)((dval * 800 * cifCurReadStyle->crs_multiplier)
+				/ cifCurReadStyle->crs_scaleFactor);
+	}
+	else
+	    UNREADRH(nbytes, rtype);
+
+	READRH(nbytes, rtype);
+	if (nbytes > 0 && rtype == CALMA_ANGLE)
+	{
+	    calmaReadR8(&dval);
+	    angle = (int)dval;
+	}
+	else
+	    UNREADRH(nbytes, rtype);
+    }
+    else
+	UNREADRH(nbytes, rtype);
+
 
     /* Coordinates of text */
     READRH(nbytes, rtype)
@@ -754,7 +833,11 @@ calmaElementText()
     /* Place the label */
     if (type >= 0)
     {
-    	 (void) DBPutLabel(cifReadCellDef, &r, -1, textbody, type, 0);
+	if (font < 0)
+	    DBPutLabel(cifReadCellDef, &r, pos, textbody, type, 0);
+	else
+	    DBPutFontLabel(cifReadCellDef, &r, font, size, angle,
+			&GeoOrigin, pos, textbody, type, 0);
     }
 
     /* done with textbody */
