@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFsee.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFsee.c,v 1.3 2008/12/06 01:00:08 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -43,12 +43,158 @@ static CellDef *cifSeeDef;
 /* Verbosity of warning messages */
 global int CIFWarningLevel = CIF_WARN_DEFAULT;
 
+typedef struct {
+   CellDef *paintDef;
+   int	   layer;
+} PaintLayerData;
+
 typedef struct {
    char *text;
    int  layer;
    int  style;
 } SeeLayerData;
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * cifPaintDBFunc --
+ *
+ *	This routine paints CIF back into the database at the inverse
+ *	scale at which it was generated.  Otherwise, it is very much like
+ *	cifPaintCurrentFunc() in CIFrdcl.c.
+ *
+ * Results:
+ *	Always returns 0 to keep the search alive.
+ *
+ * Side effects:
+ *	Paint is generated in the cell.
+ * ----------------------------------------------------------------------------
+ */
+
+int
+cifPaintDBFunc(tile, pld)
+    Tile *tile;			/* Tile of CIF information. */
+    PaintLayerData *pld;
+{
+    Rect area;
+    int pNum;
+    TileType type = pld->layer;		/* Magic type to be painted.	*/
+    CellDef *paintDef = pld->paintDef;	/* Cell to draw into.		*/
+    int cifScale = CIFCurStyle->cs_scaleFactor;
+    PaintUndoInfo ui;
+
+    /* Compute the area of the CIF tile, then scale it into
+     * Magic coordinates.
+     */
+
+    TiToRect(tile, &area);
+    area.r_xtop /= cifScale;
+    area.r_xbot /= cifScale;
+    area.r_ytop /= cifScale;
+    area.r_ybot /= cifScale;
+
+    /* Check for degenerate areas (from rescale limiting) before painting */
+    if ((area.r_xbot == area.r_xtop) || (area.r_ybot == area.r_ytop))
+	return 0;
+
+    ui.pu_def = paintDef;
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+	if (DBPaintOnPlane(type, pNum))
+	{
+	    ui.pu_pNum = pNum;
+#ifdef NONMANHATTAN
+	    DBNMPaintPlane(paintDef->cd_planes[pNum], TiGetTypeExact(tile),
+		    &area, DBStdPaintTbl(type, pNum), (PaintUndoInfo *) &ui);
+#else
+	    DBPaintPlane(paintDef->cd_planes[pNum], &area,
+		    DBStdPaintTbl(type, pNum), (PaintUndoInfo *) &ui);
+#endif
+	}
+
+    return  0;		/* To keep the search alive. */
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CIFPaintLayer --
+ *
+ * 	Generates CIF over a given area of a given cell, then
+ *	paints the CIF layer back into the cell (or optionally,
+ *	into a separate cell) as a magic layer.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Highlight information is drawn on the screen.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CIFPaintLayer(rootDef, area, cifLayer, magicLayer, paintDef)
+    CellDef *rootDef;		/* Cell for which to generate CIF.  Must be
+				 * the rootDef of a window.
+				 */
+    Rect *area;			/* Area in which to generate CIF. */
+    char *cifLayer;		/* CIF layer to highlight on the screen. */
+    int	  magicLayer;		/* Magic layer to paint with the result */
+    CellDef *paintDef;		/* CellDef to paint into (may be NULL)	*/
+{
+    int oldCount, i;
+    char msg[100];
+    SearchContext scx;
+    PaintLayerData pld;
+    TileTypeBitMask mask;
+
+    /* Make sure the desired layer exists. */
+
+    if (!CIFNameToMask(cifLayer, &mask))
+	return;
+
+    /* Paint directly into the root cellDef if passed a NULL paintDef*/
+    pld.paintDef = (paintDef == NULL) ? rootDef : paintDef;
+    pld.layer = magicLayer;
+
+    /* Flatten the area and generate CIF for it. */
+
+    CIFErrorDef = rootDef;
+    CIFInitCells();
+    UndoDisable();
+    CIFDummyUse->cu_def = rootDef;
+    GEO_EXPAND(area, CIFCurStyle->cs_radius, &scx.scx_area);
+    scx.scx_use = CIFDummyUse;
+    scx.scx_trans = GeoIdentityTransform;
+    (void) DBTreeSrTiles(&scx, &DBAllButSpaceAndDRCBits, 0,
+	cifHierCopyFunc, (ClientData) CIFComponentDef);
+    oldCount = DBWFeedbackCount;
+    CIFGen(CIFComponentDef, area, CIFPlanes, &DBAllTypeBits, TRUE, TRUE);
+    DBCellClearDef(CIFComponentDef);
+
+    /* Report any errors that occurred. */
+
+    if (DBWFeedbackCount != oldCount)
+    {
+	TxPrintf("%d problems occurred.  See feedback entries.\n",
+	    DBWFeedbackCount-oldCount);
+    }
+
+    /* Paint back the chosen layer. */
+
+    UndoEnable();
+    for (i = 0; i < CIFCurStyle->cs_nLayers; i++)
+    {
+	if (!TTMaskHasType(&mask, i)) continue;
+	(void) DBSrPaintArea((Tile *)NULL, CIFPlanes[i], &TiPlaneRect,
+	    	&CIFSolidBits, cifPaintDBFunc, (ClientData)&pld);
+    }
+
+    DBWAreaChanged(rootDef, area, DBW_ALLWINDOWS, &mask);
+    DBReComputeBbox(rootDef);
+    DRCCheckThis(rootDef, TT_CHECKPAINT, area);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -95,7 +241,7 @@ cifSeeFunc(tile, sld)
 #endif
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
