@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFtech.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFtech.c,v 1.5 2008/12/04 17:10:29 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -99,8 +99,18 @@ cifTechFreeStyle()
 		{
 		    if (op->co_client != (ClientData)NULL)
 		    {
-			if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
-			    freeMagic((char *)op->co_client);
+			switch (op->co_opcode)
+			{
+			    case CIFOP_OR:
+			    case CIFOP_BBOX:
+			    case CIFOP_MAXRECT:
+				/* These options use co_client to hold a single	*/
+				/* integer value, so it is not allocated.	*/
+				break;
+			    default:
+				freeMagic((char *)op->co_client);
+				break;
+			}
 		    }
 		    freeMagic((char *)op);
 		}
@@ -211,7 +221,7 @@ cifParseLayers(string, style, paintMask, cifMask,spaceOK)
     				   layer?
 				*/
 {
-    TileTypeBitMask curCifMask,curPaintMask;
+    TileTypeBitMask curCifMask, curPaintMask;
     char curLayer[40], *p, *cp;
     TileType paintType;
     int i;
@@ -284,14 +294,18 @@ okpaint:
 	    else
 	    {
 		TileType rtype;
+		TileTypeBitMask *rMask;
 
 		TTMaskSetMask(paintMask, &curPaintMask);
 
 		/* Add residues from '*' notation */
 		if (allResidues)
 		    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
-			if (TTMaskHasType(DBResidueMask(rtype), paintType))
+		    {
+			rMask = DBResidueMask(rtype);
+			if (TTMaskHasType(rMask, paintType))
 			    TTMaskSetType(paintMask, rtype);
+		    }
 	    } 
 	}
 	else if (!TTMaskEqual(&curCifMask, &DBZeroTypeBits))
@@ -949,6 +963,10 @@ CIFTechLine(sectionName, argc, argv)
 		CIFCurStyle->cs_flags |= CWF_PERMISSIVE_LABELS;
 	    else if (strcmp(argv[i], "grow-euclidean") == 0)
 		CIFCurStyle->cs_flags |= CWF_GROW_EUCLIDEAN;
+	    else if (strcmp(argv[i], "see-vendor") == 0)
+		CIFCurStyle->cs_flags |= CWF_SEE_VENDOR;
+	    else if (strcmp(argv[i], "no-errors") == 0)
+		CIFCurStyle->cs_flags |= CWF_NO_ERRORS;
 	}
 	return TRUE;
     }
@@ -998,6 +1016,10 @@ CIFTechLine(sectionName, argc, argv)
 	newOp->co_opcode = CIFOP_SLOTS;
     else if (strcmp(argv[0], "bbox") == 0)
 	newOp->co_opcode = CIFOP_BBOX;
+    else if (strcmp(argv[0], "net") == 0)
+	newOp->co_opcode = CIFOP_NET;
+    else if (strcmp(argv[0], "maxrect") == 0)
+	newOp->co_opcode = CIFOP_MAXRECT;
     else
     {
 	TechError("Unknown statement \"%s\".\n", argv[0]);
@@ -1115,6 +1137,26 @@ bloatCheck:
 	    TechError("Not all bloat layers fall in the same plane.\n");
 	    bloatDone: break;
 
+	case CIFOP_NET:
+	    if (argc != 3) goto wrongNumArgs;
+	    newOp->co_client = (ClientData)StrDup((char **)NULL, argv[1]);
+	    cifParseLayers(argv[2], CIFCurStyle, &newOp->co_paintMask,
+		&newOp->co_cifMask, FALSE);
+	    break;
+
+	case CIFOP_MAXRECT:
+	    if (argc == 2)
+	    {
+		if (!strncmp(argv[1], "ext", 3))
+		    newOp->co_client = (ClientData)1;
+		else if (strncmp(argv[1], "int", 3))
+		    TechError("Maxrect takes only one optional argument "
+				"\"external\" or \"internal\" (default).\n");
+	    }
+	    else if (argc != 1)
+		goto wrongNumArgs;
+	    break;
+
 	case CIFOP_BBOX:
 	    if (argc == 2)
 	    {
@@ -1675,17 +1717,28 @@ CIFTechFinal()
 		}
 		/* Presence of op->co_opcode in CIFOP_OR indicates a copy */
 		/* of the SquaresData pointer from a following operator	  */
-		/* CIFOP_BBOX uses the co_client field as a flag field.	  */
-		else if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+		/* CIFOP_BBOX and CIFOP_MAXRECT uses the co_client field  */
+		/* as a flag field, while CIFOP_NET uses it for a string. */
+		else
 		{
-		    bloats = (BloatData *)op->co_client;
-		    for (j = 0; j < TT_MAXTYPES; j++)
+		    switch (op->co_opcode)
 		    {
-			if (bloats->bl_distance[j] != 0)
-			{
-			    c = FindGCF(style->cs_scaleFactor, bloats->bl_distance[j]);
-			    if (c < minReduce) minReduce = c;
-			}
+			case CIFOP_OR:
+			case CIFOP_BBOX:
+			case CIFOP_MAXRECT:
+			case CIFOP_NET:
+			    break;
+			default:
+			    bloats = (BloatData *)op->co_client;
+			    for (j = 0; j < TT_MAXTYPES; j++)
+			    {
+				if (bloats->bl_distance[j] != 0)
+				{
+				    c = FindGCF(style->cs_scaleFactor,
+						bloats->bl_distance[j]);
+				    if (c < minReduce) minReduce = c;
+				}
+			    }
 		    }
 		}
 	    }
@@ -1887,6 +1940,12 @@ CIFGetContactSize(type, edge, spacing, border)
     int i;
     SquaresData *squares;
 
+    if (style == NULL)
+    {
+	edge = spacing = border = 0;
+	return 0;
+    }
+
     for (i = 0; i < style->cs_nLayers; i++)
     {
 	for (op = style->cs_layers[i]->cl_ops; (op != NULL) &&
@@ -2105,18 +2164,27 @@ CIFTechOutputScale(n, d)
 		/* Presence of op->co_opcode in CIFOP_OR indicates a copy */
 		/* of the SquaresData pointer from a following operator	  */
 		/* CIFOP_BBOX uses the co_client field as a flag field.	  */
-		else if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+		else
 		{
-		    bloats = (BloatData *)op->co_client;
-		    for (j = 0; j < TT_MAXTYPES; j++)
+		    switch (op->co_opcode)
 		    {
-			if (bloats->bl_distance[j] != 0)
-			{
-			    bloats->bl_distance[j] *= d;
-		            lgcf = FindGCF(abs(bloats->bl_distance[j]),
-					ostyle->cs_expander);
-			    lexpand = FindGCF(lexpand, lgcf);
-			}
+			case CIFOP_OR:
+			case CIFOP_BBOX:
+			case CIFOP_MAXRECT:
+			case CIFOP_NET:
+			    break;
+			default:
+			    bloats = (BloatData *)op->co_client;
+			    for (j = 0; j < TT_MAXTYPES; j++)
+			    {
+				if (bloats->bl_distance[j] != 0)
+				{
+				    bloats->bl_distance[j] *= d;
+		        	    lgcf = FindGCF(abs(bloats->bl_distance[j]),
+						ostyle->cs_expander);
+				    lexpand = FindGCF(lexpand, lgcf);
+				}
+			    }
 		    }
 		}
 	    }
@@ -2199,7 +2267,8 @@ CIFTechOutputScale(n, d)
 			break;
 		    default:
 			/* op->co_opcode in CIFOP_OR is a pointer copy	*/
-			/* and in CIFOP_BBOX is a flag.			*/
+			/* and in CIFOP_BBOX and CIFOP_MAXRECT is a	*/
+			/* flag, and in CIFOP_NET is a string.		*/
 			break;
 		}
 	    }
