@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFwrite.c,v 1.4 2001/03/29 19:50:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFwrite.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdlib.h>
@@ -84,6 +84,11 @@ bool CIFDoCellIdLabels = TRUE;
 
 /* prefix string for pathnames */
 char *CIFPathPrefix = NULL;
+
+/* These are normally FALSE---enable at your own risk! */
+bool CIFHierWriteDisable = FALSE;
+bool CIFArrayWriteDisable = FALSE;
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -376,8 +381,10 @@ cifOutFunc(def, f)
     GEO_EXPAND(&def->cd_bbox, CIFCurStyle->cs_radius, &bigArea);
     CIFErrorDef = def;
     CIFGen(def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE);
-    CIFGenSubcells(def, &bigArea, CIFPlanes);
-    CIFGenArrays(def, &bigArea, CIFPlanes);
+    if (!CIFHierWriteDisable)
+	CIFGenSubcells(def, &bigArea, CIFPlanes);
+    if (!CIFArrayWriteDisable)
+	CIFGenArrays(def, &bigArea, CIFPlanes);
     for (type = 0; type < CIFCurStyle->cs_nLayers; type++)
     {
 	layer = CIFCurStyle->cs_layers[type];
