@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdCD.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdCD.c,v 1.3 2008/12/03 14:12:08 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -877,16 +877,17 @@ badusage:
 #define CIF_LAMBDA	 7
 #define CIF_LIMIT	 8
 #define OSTYLE		 9
-#define PREFIX		10
-#define READ		11
-#define RESCALE		12
-#define SEE		13
-#define STATS		14
-#define WARNING		15
-#define CIF_WRITE	16
-#define CIF_WRITE_FLAT	17
+#define CIF_PAINT	10
+#define PREFIX		11
+#define READ		12
+#define RESCALE		13
+#define SEE		14
+#define STATS		15
+#define WARNING		16
+#define CIF_WRITE	17
+#define CIF_WRITE_FLAT	18
 #ifdef NONMANHATTAN
-#define POLYGONS	18
+#define POLYGONS	19
 #endif
 
 #define CIF_WARN_HELP  CIF_WARN_END	/* undefined by CIF module */
@@ -898,8 +899,10 @@ CmdCif(w, cmd)
 {
     int option, yesno;
     char **msg, *namep;
-    CellDef *rootDef;
+    CellDef *rootDef, *paintDef;
     Rect box;
+    TileType layer;
+    TileTypeBitMask mask;
     FILE *f;
     bool wizardHelp;
     bool flatCif = FALSE;
@@ -925,6 +928,8 @@ CmdCif(w, cmd)
 	"lambda in|out	show microns per lambda for the current style",
 	"limit [value]	limit internal grid subdivision by scalefactor value",
 	"ostyle [style]	change style for writing CIF to style",
+	"paint ciflayer magiclayer [cellname]\n"
+	"			generate CIF and paint into cell",
 	"prefix [path]	prepend path to cell names in CIF output",
 	"read file		read CIF from \"file\" into edit cell",
 	"rescale [yes|no]	allow/disallow rescaling of internal grid",
@@ -1250,6 +1255,38 @@ CmdCif(w, cmd)
 	    }
 	    CIFSeeLayer(rootDef, &box, argv[2]);
 	    return;
+
+	case CIF_PAINT:
+	    if ((argc != 4) && (argc != 5)) goto wrongNumArgs;
+	    if (!ToolGetBox(&rootDef, &box))
+	    {
+		TxError("Use the box to select the area in");
+		TxError(" which you want to see CIF.\n");
+		return;
+	    }
+	    if (argc == 5)
+	    {
+		paintDef = DBCellLookDef(argv[4]);
+		if (paintDef == (CellDef *)NULL)
+		{
+		    paintDef = DBCellNewDef(argv[4], (char *)NULL);
+		    DBCellSetAvail(paintDef);
+		}
+	    }
+	    else
+		paintDef = rootDef;
+
+	    layer = DBTechNoisyNameType(argv[3]);
+	    if (layer >= TT_TECHDEPBASE)
+	    {
+		CIFPaintLayer(rootDef, &box, argv[2], layer, paintDef);
+
+		/* Refresh the layout drawing */
+		TTMaskSetOnlyType(&mask, layer);
+		DBWAreaChanged(paintDef, &box, DBW_ALLWINDOWS, &mask);
+		DRCCheckThis(paintDef, TT_CHECKPAINT, &box);
+	    }
+	    return;
 	
 	case STATS:
 	    CIFPrintStats();
@@ -1471,6 +1508,7 @@ typedef struct CCS
     TileTypeBitMask *rmask;
     TileType tstart;
     Rect area;
+    Rect clip;
     LinkedRect *lhead;
 } CCStruct;
 
@@ -1520,6 +1558,7 @@ CmdContact(w, cmd)
 	ccs.rmask = rmask;
 	ccs.tstart = rtype;
 	ccs.rootDef = EditCellUse->cu_def;
+	ccs.clip = area;
 	TTMaskSetOnlyType(&smask, rtype);
 	DBSrPaintArea((Tile *) NULL, EditCellUse->cu_def->cd_planes[DBPlane(rtype)],
 		&area, &smask, cmdContactFunc, (ClientData) &ccs);
@@ -1554,6 +1593,7 @@ cmdContactFunc(tile, ccs)
     int cmdContactFunc2();	/* Forward declaration */
 
     TiToRect(tile, &ccs->area);
+    GeoClip(&ccs->area, &ccs->clip);
 
     for (stype = ccs->tstart + 1; stype < DBNumUserLayers; stype++)
 	if (TTMaskHasType(ccs->rmask, stype))
@@ -1613,6 +1653,7 @@ CmdCopy(w, cmd)
     Rect rootBox, newBox;
     Point rootPoint, editPoint;
     CellDef *rootDef;
+    int argpos;
 
     if (cmd->tx_argc > 4)
     {
@@ -1639,37 +1680,34 @@ CmdCopy(w, cmd)
 	    goto copyToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
-	if (indx < 0)
-	    return;
-	if (cmd->tx_argc == 3)
+	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, FALSE);
+	argpos = (indx < 0) ? 1 : 2;
+
+	if (cmd->tx_argc >= 3)
 	{
-	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    switch (indx)
 	    {
-		amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-		amounty = amountx;
-	    }
-	    else
-	    {
-		switch (indx)
-		{
-		    case GEO_EAST: case GEO_WEST:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			amounty = amountx;
-			break;
-		    case GEO_NORTH: case GEO_SOUTH:
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			amountx = amounty;
-			break;
-		    default:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			break;
-		}
+		case GEO_EAST: case GEO_WEST:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, TRUE);
+		    amounty = 0;
+		    break;
+		case GEO_NORTH: case GEO_SOUTH:
+		    amountx = 0;
+		    amounty = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, FALSE);
+		    break;
+		default:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, TRUE);
+		    amounty = cmdParseCoord(w, cmd->tx_argv[cmd->tx_argc - 1],
+				TRUE, FALSE);
+		    break;
 	    }
 	}
 	else
 	{
+	    if (indx < 0) {
+		TxError("Improperly defined copy. . . direction needed.\n");
+		return;
+	    }
 	    amountx = cmdParseCoord(w, "1l", TRUE, TRUE);
 	    amounty = cmdParseCoord(w, "1l", TRUE, FALSE);
 	}
@@ -1697,6 +1735,7 @@ CmdCopy(w, cmd)
 		ydelta = amounty;
 		break;
 	    case GEO_NORTHEAST:
+	    case -2:
 		xdelta = amountx;
 		ydelta = amounty;
 		break;
