@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdLQ.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdLQ.c,v 1.8 2008/12/11 18:36:43 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -137,7 +137,6 @@ CmdLabelProc(text, font, size, rotate, offx, offy, pos, type)
  *	label <text> [<direction> [<layer>]]
  *	label <text> <font> [<size> [<rotation>
  *			[<offsetx> <offsety> [<direction> [<layer>]]]]]
- *	label font <filename>
  *
  * Direction may be one of:
  *	right left top bottom
@@ -162,6 +161,7 @@ CmdLabelProc(text, font, size, rotate, offx, offy, pos, type)
  * ----------------------------------------------------------------------------
  */
 
+
 void
 CmdLabel(w, cmd)
     MagWindow *w;
@@ -169,6 +169,7 @@ CmdLabel(w, cmd)
 {
     TileType type;
     int pos, font = -1, size = 0, rotate = 0, offx = 0, offy = 0;
+    int option;
     char *p;
 
     if (cmd->tx_argc < 2 || cmd->tx_argc > 9)
@@ -179,32 +180,6 @@ CmdLabel(w, cmd)
 	return;
     }
 
-    /*
-     * Check for font load command
-     */
-    if ((cmd->tx_argc == 3) && !strcmp(cmd->tx_argv[1], "font"))
-    {
-	if (StrIsInt(cmd->tx_argv[2]))
-	{
-	    int font = atoi(cmd->tx_argv[2]);
-	    if (font < 0 || font >= DBNumFonts)
-	    {
-		if (DBNumFonts == 0)
-		    TxError("No vector outline fonts are loaded.\n");
-		else
-		    TxError("Font index out of range (0 to %d)\n", DBNumFonts - 1);
-	    }
-	    else
-		TxPrintf("%s\n", DBFontList[font]->mf_name);
-	}
-	else
-	{
-	    if (DBLoadFont(cmd->tx_argv[2]) == 0)
-		TxPrintf("%s\n", DBFontList[DBNumFonts - 1]->mf_name);
-	}
-	return;
-    }
-
     p = cmd->tx_argv[1];
 
     /*
@@ -232,7 +207,7 @@ CmdLabel(w, cmd)
 		/* Assume that this is a font name */
 
 		font = DBNameToFont(cmd->tx_argv[2]);
-		if (font < 0)
+		if (font < -1)
 		{
 		    TxError("Unknown vector outline font \"%s\"\n", cmd->tx_argv[2]);
 		    return;
@@ -246,10 +221,10 @@ CmdLabel(w, cmd)
 
     if (font >= 0)
     {
-	size = 1;
+	size = DBLambda[1];
 	if (cmd->tx_argc > 3)
-	    if (StrIsInt(cmd->tx_argv[3]))
-		size = atoi(cmd->tx_argv[3]);
+	    if (StrIsNumeric(cmd->tx_argv[3]))
+		size = cmdScaleCoord(w, cmd->tx_argv[3], TRUE, TRUE, 8);
 
 	if (cmd->tx_argc > 4)
 	    if (StrIsInt(cmd->tx_argv[4]))
@@ -261,8 +236,8 @@ CmdLabel(w, cmd)
 		/* offx and offy are entered in lambda units and	*/
 		/* re-interpreted as quarter-internal units.		*/
 
-		offx = cmdScaleCoord(w, cmd->tx_argv[5], FALSE, TRUE, 4);
-		offy = cmdScaleCoord(w, cmd->tx_argv[6], FALSE, FALSE, 4);
+		offx = cmdScaleCoord(w, cmd->tx_argv[5], TRUE, TRUE, 4);
+		offy = cmdScaleCoord(w, cmd->tx_argv[6], TRUE, FALSE, 4);
 	    }
 
 	if (cmd->tx_argc > 7)
@@ -460,6 +435,7 @@ CmdMove(w, cmd)
     Rect rootBox, newBox;
     Point rootPoint, editPoint;
     CellDef *rootDef;
+    int argpos;
 
     if (cmd->tx_argc > 4)
     {
@@ -486,32 +462,26 @@ CmdMove(w, cmd)
 	    goto moveToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
-	if (indx < 0)
-	    return;
-	if (cmd->tx_argc == 3)
+	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, FALSE);
+	argpos = (indx < 0) ? 1 : 2;
+
+	if (cmd->tx_argc >= 3)
 	{
-	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    switch (indx)
 	    {
-		amountx = amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-	    }
-	    else
-	    {
-		switch (indx)
-		{
-		    case GEO_EAST: case GEO_WEST:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			amounty = amountx;
-			break;
-		    case GEO_NORTH: case GEO_SOUTH:
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			amountx = amounty;
-			break;
-		    default:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			break;
-		}
+		case GEO_EAST: case GEO_WEST:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, TRUE);
+		    amounty = 0;
+		    break;
+		case GEO_NORTH: case GEO_SOUTH:
+		    amountx = 0;
+		    amounty = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, FALSE);
+		    break;
+		default:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, TRUE);
+		    amounty = cmdParseCoord(w, cmd->tx_argv[cmd->tx_argc - 1],
+				TRUE, FALSE);
+		    break;
 	    }
 	}
 	else
@@ -539,6 +509,7 @@ CmdMove(w, cmd)
 		ydelta = 0;
 		break;
 	    case GEO_NORTHEAST:
+	    case -2:
 		xdelta = amountx;
 		ydelta = amounty;
 		break;
@@ -704,7 +675,7 @@ CmdPaintEraseButton(w, butPoint, isPaint)
     Point *butPoint;	/* Screen location at which button was raised */
     bool isPaint;	/* True for paint, False for erase.	*/
 {
-    Rect rootRect, editRect;
+    Rect rootRect, editRect, areaReturn;
     TileTypeBitMask mask;
     DBWclientRec *crec;
 
@@ -726,6 +697,7 @@ CmdPaintEraseButton(w, butPoint, isPaint)
 
     if (!ToolGetEditBox(&editRect)) return;
 
+    areaReturn = editRect;
     if (TTMaskEqual(&mask, &DBZeroTypeBits))
     {
 	TileTypeBitMask eraseMask;
@@ -743,7 +715,7 @@ CmdPaintEraseButton(w, butPoint, isPaint)
 
 	TTMaskAndMask3(&eraseMask, &crec->dbw_visibleLayers, &DBActiveLayerBits);
 	DBEraseValid(EditCellUse->cu_def, &editRect, &eraseMask, 0);
-	(void) DBEraseLabel(EditCellUse->cu_def, &editRect, &mask);
+	DBEraseLabel(EditCellUse->cu_def, &editRect, &mask, &areaReturn);
     }
     else if (isPaint)
 	DBPaintValid(EditCellUse->cu_def, &editRect, &mask, 0);
@@ -756,7 +728,7 @@ CmdPaintEraseButton(w, butPoint, isPaint)
     DBAdjustLabels(EditCellUse->cu_def, &editRect);
 
     DRCCheckThis (EditCellUse->cu_def, TT_CHECKPAINT, &editRect);
-    DBWAreaChanged(EditCellUse->cu_def, &editRect, DBW_ALLWINDOWS, &mask);
+    DBWAreaChanged(EditCellUse->cu_def, &areaReturn, DBW_ALLWINDOWS, &mask);
     DBReComputeBbox(EditCellUse->cu_def);
     UndoNext();
 }
