@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdTZ.c,v 1.2 2008/02/07 06:25:25 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdTZ.c,v 1.5 2008/09/05 13:56:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -247,7 +247,7 @@ CmdTech(w, cmd)
 	case TECH_LAYERS:
 	    if (cmd->tx_argc == 4)
 	    {
-		TileTypeBitMask lockmask;
+		TileTypeBitMask lockmask, *rMask;
 		TileType ctype;
 
 		if (!strcmp(cmd->tx_argv[3], "*"))
@@ -262,11 +262,14 @@ CmdTech(w, cmd)
 			    if (DBIsContact(ctype))
 				DBLockContact(ctype);
 		    for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
-			if (TTMaskIntersect(&lockmask, DBResidueMask(ctype)))
+		    {
+			rMask = DBResidueMask(ctype);
+			if (TTMaskIntersect(&lockmask, rMask))
 			{
 			    TTMaskClearType(&DBActiveLayerBits, ctype);
 			    DBLockContact(ctype);
 			}
+		    }
 		}
 		else if (!strcmp(cmd->tx_argv[2], "unlock"))
 		{
@@ -279,9 +282,9 @@ CmdTech(w, cmd)
 		    for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
 		    {
 			TileTypeBitMask testmask;
-			TTMaskAndMask3(&testmask, &DBActiveLayerBits,
-					DBResidueMask(ctype));
-			if (TTMaskEqual(&testmask, DBResidueMask(ctype)))
+			rMask = DBResidueMask(ctype);
+			TTMaskAndMask3(&testmask, &DBActiveLayerBits, rMask);
+			if (TTMaskEqual(&testmask, rMask))
 			{
 			    TTMaskSetType(&DBActiveLayerBits, ctype);
 			    DBUnlockContact(ctype);
@@ -753,24 +756,31 @@ CmdUpsidedown(w, cmd)
  * cmdWhatPrintCell --
  *
  * This command is used to print the cell id of the types encountered in the
- * what command. For each type it only prints the name once.  The routine
- * must be initialized by calling it with NULL args everytime the tile type
- * changes and we start a new search.  (BSI)
+ * what command. For each type it only prints the name once.
  *
  * Results:
  *	Returns 0 to keep the search going.
  *
  * Side Effects:
- *	Prints cell names to the terminal.
+ *	Prints cell names to the terminal.  Adds to a maintained list of
+ *	unique cell IDs, which needs to be free'd by the calling routine.
  * ----------------------------------------------------------------------------
  */
 
+struct linked_id {
+   char *lid_name;
+   struct linked_id *lid_next;
+};
+
+
 int cmdWhatPrintCell(tile, cxp)
    Tile *tile;
    TreeContext *cxp;
 {
+    struct linked_id **lid = (struct linked_id **)cxp->tc_filter->tf_arg;
+    struct linked_id *curlid = *lid;
     char *CurrCellName;
-    char **whatCell = (char **)cxp->tc_filter->tf_arg;
+    char *whatCell;
 
     // Use the id of the cell or its name if top cell
 
@@ -778,10 +788,19 @@ int cmdWhatPrintCell(tile, cxp)
     if ((CurrCellName == NULL) || (CurrCellName[0] == '\0'))
 	CurrCellName = cxp->tc_scx->scx_use->cu_def->cd_name;
 
-    if (*whatCell != CurrCellName)
+    for (curlid = *lid; curlid != NULL; curlid = curlid->lid_next)
+    {
+	whatCell = curlid->lid_name;
+	if (whatCell == CurrCellName)
+	    break;
+    }
+    if (curlid == NULL)
     {
 	TxPrintf(" %s ", CurrCellName);
-	*whatCell = CurrCellName;
+	curlid = (struct linked_id *)mallocMagic(sizeof(struct linked_id));
+	curlid->lid_name = CurrCellName;
+	curlid->lid_next = *lid;
+	*lid = curlid;
     }
     return 0;
 }
@@ -826,10 +845,10 @@ CmdWhat(w, cmd)
     int i, locargc;
     bool foundAny;
     bool doList = FALSE;
-    TileTypeBitMask layers, maskBits;
+    TileTypeBitMask layers, maskBits, *rMask;
     SearchContext scx;
     CellUse *CheckUse;
-    char *whatCell;
+    struct linked_id *lid;
 
 #ifdef MAGIC_WRAPPER
     Tcl_Obj *lobj, *paintobj, *labelobj, *cellobj;
@@ -879,7 +898,10 @@ CmdWhat(w, cmd)
 	for (i = DBNumUserLayers; i < DBNumTypes; i++)
 	{
 	    if (TTMaskHasType(&layers, i))
-		TTMaskSetMask(&layers, DBResidueMask(i));
+	    {
+		rMask = DBResidueMask(i);
+		TTMaskSetMask(&layers, rMask);
+	    }
 	    TTMaskClearType(&layers, i);
 	}
     }
@@ -917,13 +939,18 @@ CmdWhat(w, cmd)
 		{
 		    if (TTMaskHasType(&layers, i))
 		    {
-			whatCell = NULL;
+			lid = NULL;
 			TxPrintf("    %-8s (", DBTypeLongName(i));
 			TTMaskSetOnlyType(&maskBits, i);
 			if (DBIsContact(i)) DBMaskAddStacking(&maskBits);
 			DBTreeSrTiles(&scx, &maskBits, 0, cmdWhatPrintCell,
-				(ClientData)&whatCell);
+				(ClientData)&lid);
 			TxPrintf(")\n");
+			while (lid != NULL)
+			{
+			    freeMagic(lid);
+			    lid = lid->lid_next;
+			}
 		    }
 		}
 	    }
@@ -1229,8 +1256,10 @@ cmdWhatCellListFunc(selUse, realUse, transform, newobj)
 #define SHOW		5
 #define SWITCH		6
 #define TYPE		7
-#define VERTICAL	8
-#define SEGMENT		9
+#define VALUES		8
+#define VERTICAL	9
+#define WIDTH		10
+#define SEGMENT		11
 
 void
 CmdWire(w, cmd)
@@ -1242,6 +1271,10 @@ CmdWire(w, cmd)
     TileType type;
     int width;
 
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
     static char *cmdWireOption[] =
     {	
 	"decrement layer|width 	decrement the wire layer or width",
@@ -1251,8 +1284,10 @@ CmdWire(w, cmd)
 	"leg                    add a new horizontal or vertical leg",
 	"show			show next wire segment as a selection",
 	"switch [layer width]   place contact and switch layers",
-	"type [layer width]     select the type and size of wires",
+	"type [layer [width]]   select the type and size of wires",
+	"values			query current wire type and width",
 	"vertical               add a new vertical wire leg",
+	"width [value]		change the width of the wire",
 	"segment layer width x1 y1 x2 y2... [-noendcap]  (or)\n"
 	"segment layer width filename [-noendcap]\n"
 	"			paint one or more wire segments",
@@ -1279,7 +1314,7 @@ CmdWire(w, cmd)
     switch (option)
     {
 	case DECREMENT:
-	    if (cmd->tx_argc != 3)
+	    if (cmd->tx_argc != 3 && cmd->tx_argc != 4)
 		goto badargs;
 	    if (!strcmp(cmd->tx_argv[2], "type"))
 	    {
@@ -1308,13 +1343,18 @@ CmdWire(w, cmd)
 		}
 	    }
 	    else if (!strcmp(cmd->tx_argv[2], "width"))
-		WirePickType(WireType, WireWidth - 1);
+	    {
+		int value = 1;
+		if (cmd->tx_argc == 4)
+		    value = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
+		WirePickType(WireType, WireWidth - value);
+	    }
 	    else
 		goto badargs;
 	    return;
 
 	case INCREMENT:
-	    if (cmd->tx_argc != 3)
+	    if (cmd->tx_argc != 3 && cmd->tx_argc != 4)
 		goto badargs;
 	    if (!strcmp(cmd->tx_argv[2], "type"))
 	    {
@@ -1343,7 +1383,12 @@ CmdWire(w, cmd)
 		}
 	    }
 	    else if (!strcmp(cmd->tx_argv[2], "width"))
-		WirePickType(WireType, WireWidth + 1);
+	    {
+		int value = 1;
+		if (cmd->tx_argc == 4)
+		    value = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
+		WirePickType(WireType, WireWidth + value);
+	    }
 	    else
 		goto badargs;
 	    return;
@@ -1399,7 +1444,7 @@ CmdWire(w, cmd)
 	case TYPE:
 	    if (locargc == 2)
 		WirePickType(-1, 0);
-	    else if (locargc != 4)
+	    else if (locargc != 3 && locargc != 4)
 	    {
 		badargs:
 		TxError("Wrong arguments.  The correct syntax is\n");
@@ -1421,16 +1466,64 @@ CmdWire(w, cmd)
 			    cmd->tx_argv[2]);
 		    return;
 		}
-		width = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
+		if (locargc == 3)
+		{
+		    int minwidth = DRCGetDefaultLayerWidth(type);
+		    width = WireGetWidth();
+		    if (width < minwidth) width = minwidth;
+		}
+		else
+		    width = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
 		WirePickType(type, width);
 		return;
 	    }
 	    break;
 
+	case VALUES:
+	    if (locargc == 2)
+	    {
+		width = WireGetWidth();
+		type = WireGetType();
+#ifdef MAGIC_WRAPPER
+		lobj = Tcl_NewListObj(0, NULL);
+		Tcl_ListObjAppendElement(magicinterp, lobj,
+				Tcl_NewIntObj(width));
+		Tcl_ListObjAppendElement(magicinterp, lobj,
+				Tcl_NewStringObj(DBTypeLongNameTbl[type], -1));
+		Tcl_SetObjResult(magicinterp, lobj);
+#else
+		TxPrintf("Wire layer %s, width %d\n",
+			DBTypeLongNameTbl[type], width);
+#endif
+	    }
+	    return;
+
 	case VERTICAL:
 	    WireAddLeg((Rect *) NULL, (Point *) NULL, WIRE_VERTICAL);
 	    return;
 
+	case WIDTH:
+	    if (locargc == 2)
+	    {
+		width = WireGetWidth();
+#ifdef MAGIC_WRAPPER
+		lobj = Tcl_NewIntObj(width);
+		Tcl_SetObjResult(magicinterp, lobj);
+#else
+		TxPrintf("Wire width is %d\n", width);
+#endif
+	    }
+	    else if (locargc != 3)
+		goto badargs;
+	    else
+	    {
+		width = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+		type =  WireGetType();
+		WirePickType(type, width);
+		return;
+	    }
+	    break;
+
 	case SEGMENT:
 	    {
 		Point *plist;
