@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.2 2008/02/07 05:44:21 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.6 2008/12/11 04:20:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -150,10 +150,11 @@ DBPaintPlaneActive(def, pNum, type, area, undo)
 
     if (DBIsContact(loctype))
     {
-	TileTypeBitMask tmask;
+	TileTypeBitMask tmask, *rMask;
 
-	TTMaskAndMask3(&tmask, DBResidueMask(loctype), &DBActiveLayerBits);
-	if (!TTMaskEqual(&tmask, DBResidueMask(loctype)))
+	rMask = DBResidueMask(loctype);
+	TTMaskAndMask3(&tmask, rMask, &DBActiveLayerBits);
+	if (!TTMaskEqual(&tmask, rMask))
 	{
 	    if (!TTMaskIsZero(&tmask))
 		for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
@@ -304,7 +305,8 @@ DBCellCopyAllLabels(scx, mask, xMask, targetUse, pArea)
 	pArea->r_xtop = -1;
     }
     (void) DBTreeSrLabels(scx, mask, xMask, (TerminalPath *) 0,
-			dbCopyAllLabels, (ClientData) &arg);
+			TF_LABEL_ATTACH, dbCopyAllLabels,
+			(ClientData) &arg);
 }
 
     /*ARGSUSED*/
@@ -316,28 +318,37 @@ dbCopyAllLabels(scx, lab, tpath, arg)
     struct copyLabelArg *arg;
 {
     Rect labTargetRect;
-    int targetPos, targetRotate;
-    Point targetOffset;
+    Point labOffset;
+    int targetPos, labRotate;
     CellDef *def;
 
     def = arg->cla_targetUse->cu_def;
     if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
     targetPos = GeoTransPos(&scx->scx_trans, lab->lab_just);
-    targetRotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
-    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &targetOffset);
+    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &labOffset);
+    labRotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
 
     /* Eliminate duplicate labels.  Don't pay any attention to layers
-     * or other properties besides text in deciding on duplicates:  if
-     * the text matches, it's a duplicate.
+     * in deciding on duplicates:  if text and position match, it's a
+     * duplicate.
      */
 
-    (void) DBEraseLabelsByContent(def, &labTargetRect, -1, lab->lab_text);
-    (void) DBPutFontLabel(def, &labTargetRect, lab->lab_font,
-		lab->lab_size, targetRotate, &targetOffset,
-		targetPos, lab->lab_text, lab->lab_type, lab->lab_flags);
+    DBEraseLabelsByContent(def, &labTargetRect, -1, lab->lab_text);
+    DBPutFontLabel(def, &labTargetRect, lab->lab_font,
+		lab->lab_size, labRotate, &labOffset, targetPos,
+		lab->lab_text, lab->lab_type, lab->lab_flags);
     if (arg->cla_bbox != NULL)
-	(void) GeoIncludeAll(&labTargetRect, arg->cla_bbox);
+    {
+	GeoIncludeAll(&labTargetRect, arg->cla_bbox);
+
+	/* Rendered font labels include the bounding box of the text itself */
+	if (lab->lab_font >= 0)
+	{
+	    GeoTransRect(&scx->scx_trans, &lab->lab_bbox, &labTargetRect);
+	    GeoIncludeAll(&labTargetRect, arg->cla_bbox);
+	}
+    }
     return 0;
 }
 
@@ -441,8 +452,8 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
     CellDef *def = targetUse->cu_def;
     Rect labTargetRect;
     Rect *rect = &scx->scx_area;
-    int targetPos, targetRotate;
-    Point targetOffset;
+    int targetPos, labRotate;
+    Point labOffset;
     CellUse *sourceUse = scx->scx_use;
 
     if (pArea != NULL)
@@ -460,21 +471,20 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
 		|| TTMaskHasType(mask, L_LABEL)))
 	{
 	    GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
-	    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &targetOffset);
 	    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_just);
-	    targetRotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
+	    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &labOffset);
+	    labRotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
+
 
 	    /* Eliminate duplicate labels.  Don't pay any attention to
 	     * type when deciding on duplicates, since types can change
 	     * later and then we'd have a duplicate.
 	     */
 
-	    (void) DBEraseLabelsByContent(def, &labTargetRect,
-		    -1, lab->lab_text);
-	    (void) DBPutFontLabel(def, &labTargetRect,
-			lab->lab_font, lab->lab_size, targetRotate,
-			&targetOffset, targetPos, lab->lab_text,
-			lab->lab_type, lab->lab_flags, TRUE);
+	    DBEraseLabelsByContent(def, &labTargetRect, -1, lab->lab_text);
+	    DBPutFontLabel(def, &labTargetRect, lab->lab_font,
+			lab->lab_size, labRotate, &labOffset, targetPos,
+			lab->lab_text, lab->lab_type, lab->lab_flags);
 	    if (pArea != NULL)
 		(void) GeoIncludeAll(&labTargetRect, pArea);
 	}
@@ -577,10 +587,11 @@ dbCopyAllPaint(tile, cxp)
     /* Resolve what type we're going to paint, based on the type and mask */
     if (!TTMaskHasType(typeMask, type))
     {
-	TileTypeBitMask rMask;
+	TileTypeBitMask rMask, *tmask;
 
 	/* Simple case---typeMask has a residue of type on pNum */
-	TTMaskAndMask3(&rMask, typeMask, DBResidueMask(type));
+	tmask = DBResidueMask(type);
+	TTMaskAndMask3(&rMask, typeMask, tmask);
 	TTMaskAndMask(&rMask, &DBPlaneTypes[pNum]);
 	if (!TTMaskIsZero(&rMask))
 	{
