@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellname.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellname.c,v 1.3 2008/12/11 18:36:43 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1059,7 +1059,7 @@ dbLockUseFunc(selUse, use, transform, data)
 {
     bool dolock = *((bool *)data);
 
-    if (!DBIsChild(use, EditCellUse))
+    if (EditCellUse && !DBIsChild(use, EditCellUse))
     {
 	TxError("Cell %s (%s) isn't a child of the edit cell.\n",
 		use->cu_id, use->cu_def->cd_name);
@@ -1124,8 +1124,11 @@ DBLockUse(UseName, bval)
 
     if (UseName == NULL)
     {
-	SelEnumCells(TRUE, (int *)NULL, (SearchContext *)NULL,
-		dbLockUseFunc, (ClientData)&bval);
+	if (EditCellUse == NULL)
+	    TxError("Cannot set lock in a non-edit cell!\n");
+	else
+	    SelEnumCells(TRUE, (int *)NULL, (SearchContext *)NULL,
+			dbLockUseFunc, (ClientData)&bval);
     }
     else
     {
@@ -1269,6 +1272,7 @@ DBCellDefAlloc()
     cellDef->cd_bbox.r_ybot = 0;
     cellDef->cd_bbox.r_xtop = 1;
     cellDef->cd_bbox.r_ytop = 1;
+    cellDef->cd_extended = cellDef->cd_bbox;
     cellDef->cd_name = (char *) NULL;
     cellDef->cd_file = (char *) NULL;
 #ifdef FILE_LOCKS
