@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellsrch.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellsrch.c,v 1.3 2008/12/11 04:20:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -490,7 +490,7 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
  */
 
 int
-DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
+DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
     SearchContext *scx;		/* Pointer to search context specifying
 				 * a cell use to search, an area in the
 				 * coordinates of the cell's def, and a
@@ -511,6 +511,11 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
 				 * new pathname components as we encounter
 				 * them.
 				 */
+    unsigned char flags;	/* Flags to denote whether labels should be
+				 * searched according to the area of the
+				 * attachment, the area of the label itself,
+				 * or both.
+				 */
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
@@ -520,6 +525,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     CellUse *cellUse = scx->scx_use;
     CellDef *def = cellUse->cu_def;
     TreeFilter filter;
+    bool is_touching;
     int dbCellLabelSrFunc();
 
     ASSERT(def != (CellDef *) NULL, "DBTreeSrLabels");
@@ -530,7 +536,13 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
     {
 	if (SigInterruptPending) break;
-	if (GEO_TOUCH(&lab->lab_rect, r) && TTMaskHasType(mask, lab->lab_type))
+	is_touching = FALSE;
+	if ((lab->lab_font < 0) || (flags & TF_LABEL_ATTACH))
+	    is_touching = GEO_TOUCH(&lab->lab_rect, r);
+	if (!is_touching && (flags & TF_LABEL_DISPLAY) && (lab->lab_font >= 0))
+	    is_touching = GEO_TOUCH(&lab->lab_bbox, r);
+
+	if (is_touching && TTMaskHasType(mask, lab->lab_type))
 	    if ((*func)(scx, lab, tpath, cdarg))
 		return (1);
     }
@@ -540,6 +552,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     filter.tf_mask = mask;
     filter.tf_xmask = xMask;
     filter.tf_tpath = tpath;
+    filter.tf_flags = flags;
     /* filter.tf_planes is unused */
 
     /* Visit each child CellUse recursively.
@@ -579,6 +592,7 @@ dbCellLabelSrFunc(scx, fp)
     CellDef *def = scx->scx_use->cu_def;
     char *tnext;
     int result;
+    bool has_overlap;
 
     ASSERT(def != (CellDef *) NULL, "dbCellLabelSrFunc");
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask)) return 0;
@@ -603,8 +617,14 @@ dbCellLabelSrFunc(scx, fp)
 
     result = 0;
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
-	if (GEO_OVERLAP(&lab->lab_rect, r)
-		&& TTMaskHasType(mask, lab->lab_type))
+    {
+	has_overlap = FALSE;
+	if ((lab->lab_font < 0) || (fp->tf_flags & TF_LABEL_ATTACH))
+	    has_overlap = GEO_OVERLAP(&lab->lab_rect, r);
+	if (!has_overlap && (fp->tf_flags & TF_LABEL_DISPLAY) && (lab->lab_font >= 0))
+	    has_overlap = GEO_OVERLAP(&lab->lab_bbox, r);
+
+	if (has_overlap && TTMaskHasType(mask, lab->lab_type))
 	{
 	    if ((*fp->tf_func)(scx, lab, fp->tf_tpath, fp->tf_arg))
 	    {
@@ -612,6 +632,7 @@ dbCellLabelSrFunc(scx, fp)
 		goto cleanup;
 	    }
 	}
+    }
 
     /* Now visit each child use recursively */
     if (DBCellSrArea(scx, dbCellLabelSrFunc, (ClientData) fp))
