@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBlabel2.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBlabel2.c,v 1.2 2008/06/01 18:37:39 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -135,8 +135,8 @@ DBSearchLabel(scx, mask, xMask, pattern, func, cdarg)
     tpath.tp_first = tpath.tp_next = labSrStr;
     tpath.tp_last = &(labSrStr[sizeof(labSrStr) - 2]);
 
-    if (DBTreeSrLabels(scx, mask, xMask, &tpath, dbSrLabelFunc,
-		(ClientData) &labsr))
+    if (DBTreeSrLabels(scx, mask, xMask, &tpath, TF_LABEL_ATTACH,
+		dbSrLabelFunc, (ClientData) &labsr))
 	return 1;
     else return 0;
 }
