@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtpaint2.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtpaint2.c,v 1.3 2008/12/03 14:12:08 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -90,7 +90,7 @@ void
 DBTechFinalCompose()
 {
     TileType i;
-    TileTypeBitMask testmask;
+    TileTypeBitMask testmask, *rMask;
 
     /* Default rules for painting/erasing contacts */
     dbComposePaintAllImages();
@@ -110,8 +110,9 @@ DBTechFinalCompose()
 		DBLockContact(i);
     for (i = DBNumUserLayers; i < DBNumTypes; i++)
     {
-	TTMaskAndMask3(&testmask, &DBActiveLayerBits, DBResidueMask(i));
-	if (!TTMaskEqual(&testmask, DBResidueMask(i)))
+	rMask = DBResidueMask(i);
+	TTMaskAndMask3(&testmask, &DBActiveLayerBits, rMask);
+	if (!TTMaskEqual(&testmask, rMask))
 	{
 	    TTMaskClearType(&DBActiveLayerBits, i);
 	    DBLockContact(i);
@@ -671,7 +672,7 @@ dbComposeEraseContact(lpImage, lpErase)
     {
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	    if (PlaneMaskHasPlane(lpImage->l_pmask, pNum))
-		SETERASE(lpImage->l_type, lpErase->l_type, pNum, lpImage->l_type);
+		SETERASE(lpImage->l_type, lpErase->l_type, pNum, TT_SPACE);
     }
     else
     {
@@ -749,9 +750,12 @@ DBLockContact(ctype)
 	/* is still valid if pc is an active layer.			*/
 
 	if (ctype >= DBNumUserLayers)
-	    if (TTMaskHasType(DBResidueMask(ctype), n))
+	{
+	    rmask = DBResidueMask(ctype);
+	    if (TTMaskHasType(rmask, n))
 		if (TTMaskHasType(&DBActiveLayerBits, n))
 		    continue;
+	}
 
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	    if (PlaneMaskHasPlane(lpPaint->l_pmask, pNum))
