@@ -20,7 +20,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCarray.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCarray.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
@@ -45,7 +45,7 @@ extern int drcArrayYankFunc(), drcArrayOverlapFunc();
 static DRCCookie drcArrayCookie = {
     0, 0, 0, 0,
     { 0 }, { 0 },
-    0, 0,
+    0, 0, 0,
     "This layer can't abut or partially overlap between array elements",
     (DRCCookie *) NULL
 };
