@@ -19,7 +19,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCsubcell.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCsubcell.c,v 1.3 2008/06/01 18:37:40 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -394,7 +394,7 @@ drcExactOverlapTile(tile, cxp)
 				 */
 {
     struct drcClientData *arg;
-    TileTypeBitMask typeMask, invMask;
+    TileTypeBitMask typeMask, invMask, *rmask;
     TileType type, t;
     Tile *tp;
     Rect r1, r2, r3, rex;
@@ -410,8 +410,11 @@ drcExactOverlapTile(tile, cxp)
     type = TiGetType(tile);
     TTMaskSetOnlyType(&typeMask, type);
     for (t = DBNumUserLayers; t < DBNumTypes; t++)
-	if (TTMaskHasType(DBResidueMask(t), type))
+    {
+	rmask = DBResidueMask(t);
+	if (TTMaskHasType(rmask, type))
 	    TTMaskSetType(&typeMask, t);
+    }
     TTMaskCom2(&invMask, &typeMask);
 
     for (i = PL_TECHDEPBASE; i < DBNumPlanes; i++)
@@ -560,7 +563,14 @@ DRCInteractionCheck(def, area, func, cdarg)
 	     * area we're interested in. */
 
 	    if (!DRCFindInteractions(def, &square, DRCTechHalo, &intArea))
+	    {
+		/* Added May 4, 2008---if there are no subcells, run the
+		 * basic check over the area of the square.
+		 */
+		GEO_EXPAND(&square, DRCTechHalo, &intArea);
+                DRCBasicCheck(def, &intArea, &square, func, cdarg); 
 		continue;
+	    }
 	    GeoClip(&intArea, area);
     
 	    /* Flatten the interaction area. */
