@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCtech.c,v 1.2 2008/02/09 17:20:36 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCtech.c,v 1.4 2008/12/23 22:32:25 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -541,6 +541,8 @@ DRCTechStyleInit()
     DRCCurStyle->DRCScaleFactorD = 1;
     DRCCurStyle->DRCStepSize = 0;
 
+    DRCTechHalo = 0;
+
     /* Put a dummy rule at the beginning of the rules table for each entry */
 
     for (i = 0; i < TT_MAXTYPES; i++)
@@ -2286,7 +2288,7 @@ drcEdge(argc, argv)
     ptest = DBTechNoisyNameMask(okTypes, &setM);
     pMaskM = CoincidentPlanes(&setM, ptest);
  
-    if (pMaskM == 0)
+    if (pMaskM == 0 || pMaskM == DBTypePlaneMaskTbl[TT_SPACE])
     {
 	/* Technically it should be illegal to specify simply "space"
 	 * in the types list for a DRC rule, as it is ambiguous.
