@@ -16,7 +16,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid $Header: /usr/cvsroot/magic-8.0/extflat/EFint.h,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-8.0/extflat/EFint.h,v 1.2 2008/12/03 14:12:09 tim Exp $
  */
 
 #ifndef _EFINT_H
@@ -264,6 +264,7 @@ extern bool efResistChanged;	/* TRUE if some cells' resistclasses unequal */
 
 extern HashTable efFreeHashTable;
 extern HashTable efNodeHashTable;
+extern HashTable efDevParamTable;
 extern HashTable efHNUseHashTable;
 extern HashTable efCapHashTable;
 extern HashTable efDistHashTable;
@@ -306,6 +307,7 @@ extern void CapHashSetValue();
 	    This should be done to avoid trouble with argument promotion
 	    that some ANSI C compilers introduce */
 
+extern DevParam *efGetDeviceParams();
 extern void efBuildNode();
 extern void efBuildConnect();
 extern void efBuildResistor();
