@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFread.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFread.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -51,9 +51,9 @@ char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
  */
 typedef enum
 {
-    ADJUST, ATTR, CAP, DEVICE, DIST, EQUIV, FET, KILLNODE, MERGE, NODE, PORT,
-    RESISTOR, RESISTCLASS, RNODE, SCALE, TECH, TIMESTAMP, USE, VERSION,
-    EXT_STYLE
+    ADJUST, ATTR, CAP, DEVICE, DIST, EQUIV, FET, KILLNODE, MERGE, NODE,
+    PARAMETERS, PORT, RESISTOR, RESISTCLASS, RNODE, SCALE, TECH,
+    TIMESTAMP, USE, VERSION, EXT_STYLE
 } Key;
 
 static struct
@@ -74,6 +74,7 @@ keyTable[] =
     "killnode",		KILLNODE,	2,
     "merge",		MERGE,		3,
     "node",		NODE,		7,
+    "parameters",	PARAMETERS,	3,
     "port",		PORT,		8,
     "resist",		RESISTOR,	4,
     "resistclasses",	RESISTCLASS,	1,
@@ -270,11 +271,6 @@ readfile:
 	    /* device res|cap|rsubckt type xlo ylo xhi yhi value	*/
 	    /*		GATE T1 T2 ...					*/
 	    case DEVICE:
-		r.r_xbot = atoi(argv[3]);
-		r.r_ybot = atoi(argv[4]);
-		r.r_xtop = atoi(argv[5]);
-		r.r_ytop = atoi(argv[6]);
-
 		/* Parse device class */
 		for (n = 0; extDevTable[n] != NULL; n++)
 		    if (!strcmp(argv[1], extDevTable[n]))
@@ -289,7 +285,6 @@ readfile:
 		{
 		    case DEV_MOSFET:
 		    case DEV_BJT:
-		    case DEV_SUBCKT:
 			ac = 10;
 			break;
 		    case DEV_DIODE:
@@ -302,31 +297,37 @@ readfile:
 			else
 			   ac = 9;	/* Has device L and W */
 			break;
+		    case DEV_SUBCKT:
 		    case DEV_RSUBCKT:
-			ac = 9;	/* Similar to RES, with a model */
-			break;
+			ac = 7;	/* Actually can have many arguments, which */
+			break;	/* we will deal with in efBuildDevice().   */
 		}
-		if (((argc - ac) % 3) != 0)
+
+		r.r_xbot = atoi(argv[3]);
+		r.r_ybot = atoi(argv[4]);
+		r.r_xtop = atoi(argv[5]);
+		r.r_ytop = atoi(argv[6]);
+
+		if (efBuildDevice(def, (char)n, argv[2], &r, argc - 7,
+				&argv[7]) != 0)
 		{
 		    efReadError("Incomplete terminal description for device\n");
 		    continue;
 		}
-		efBuildDevice(def, (char)n, argv[2], &r, argc - 7, &argv[7]);
 		break;
 
 	    /* for backwards compatibility */
 	    /* fet type xlo ylo xhi yhi area perim substrate GATE T1 T2 ... */
 	    case FET:
-		if (((argc - 9) % 3) != 0)
+		r.r_xbot = atoi(argv[2]);
+		r.r_ybot = atoi(argv[3]);
+		r.r_xtop = atoi(argv[4]);
+		r.r_ytop = atoi(argv[5]);
+		if (efBuildDevice(def, DEV_FET, argv[1], &r, argc - 6, &argv[6]) != 0)
 		{
 		    efReadError("Incomplete terminal description for fet\n");
 		    continue;
 		}
-		r.r_xbot = atoi(argv[2]);
-		r.r_ybot = atoi(argv[3]);
-		r.r_xtop = atoi(argv[4]);
-		r.r_ytop = atoi(argv[5]),
-		efBuildDevice(def, DEV_FET, argv[1], &r, argc - 6, &argv[6]);
 		break;
 
 	    /* merge node1 node2 C a1 p1 a2 p2 ... */
@@ -365,6 +366,11 @@ readfile:
 			    &argv[7], ac);
 		break;
 
+	    /* parameters name <type=string ...> */
+	    case PARAMETERS:
+		efBuildDeviceParams(argv[1], argc - 2, &argv[2]);
+		break;
+
 	    /* port name num xl yl xh yh type */
 	    case PORT:
 		if (DoSubCircuit)
