@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFvisit.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFvisit.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -153,6 +153,9 @@ efVisitSubcircuits(hc, ca)
  * EFGetLengthAndWidth --
  *
  *	Estimate length and width for a device from area and perimeter values.
+ *	Mostly this routine is meant to handle the older "fet" record.
+ *	Newer "device" types should have length and width properly determined
+ *	already, and we just return those values from the device structure.
  *
  * Results:
  *	None
@@ -177,15 +180,16 @@ EFGetLengthAndWidth(dev, lptr, wptr)
 	case DEV_MOSFET:
 	case DEV_BJT:
 	case DEV_SUBCKT:
-	case DEV_RES:
+	case DEV_RSUBCKT:
+	case DEV_DIODE:
 	case DEV_CAP:
-	    l = dev->dev_values.dev_lw.dev_length;
-	    w = dev->dev_values.dev_lw.dev_width;
+	    l = dev->dev_length;
+	    w = dev->dev_width;
 	    break;
 
 	case DEV_FET:
-	    area = dev->dev_values.dev_ap.dev_area;
-	    perim = dev->dev_values.dev_ap.dev_perim;
+	    area = dev->dev_area;
+	    perim = dev->dev_perim;
 
 	    gate = &dev->dev_terms[0];
 
@@ -214,7 +218,7 @@ EFGetLengthAndWidth(dev, lptr, wptr)
 	    break;
 
 	default:
-	    l = w = 0;	/* DIODE device shouldn't use l & w. */
+	    l = w = 0;
 	    break;
     }
 
