@@ -16,7 +16,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid $Header: /usr/cvsroot/magic-8.0/extflat/extflat.h,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-8.0/extflat/extflat.h,v 1.2 2008/12/03 14:12:09 tim Exp $
  */
 
 #ifndef _EXTFLAT_H
@@ -225,17 +225,13 @@ typedef struct devterm
  * tables of names, rather than being individually allocated for each
  * transistor.
  */
-typedef struct
-{
-    int		 dev_area;
-    int		 dev_perim;
-} apvals;
 
-typedef struct
+typedef struct parm
 {
-    int		 dev_length;
-    int		 dev_width;
-} lwvals;
+    char	 parm_type;
+    char	*parm_name;
+    struct parm	*parm_next;
+} DevParam;
 
 typedef struct dev
 {
@@ -245,12 +241,16 @@ typedef struct dev
     U_char	 dev_nterm;	/* Number of terminals in device */
     EFNode	*dev_subsnode;	/* Substrate node */
     Rect	 dev_rect;	/* 1x1 rectangle inside device */
-    union {
-        float	 dev_cap;	/* Capacitance for device class "cap" */
-        float	 dev_res;	/* Resistance for device class "res" */
-        apvals   dev_ap;	/* Total area and perimeter */
-	lwvals	 dev_lw;	/* Device length and width */
-    } 		 dev_values;	/* Which values are used depends on the class */
+
+    /* Most device types use only one or two of these, but subcircuits	*/
+    /* may keep all values to pass along as parameters.			*/
+    float	 dev_cap;	/* Capacitance for class "cap" or subckt */
+    float	 dev_res;	/* Resistance for class "res" or subckt */
+    int		 dev_area;
+    int		 dev_perim;
+    int		 dev_length;
+    int		 dev_width;
+    DevParam	 *dev_params;	/* List of subcircuit parameters to output */
     DevTerm	 dev_terms[1];	/* Terminals.  The actual number will depend
 				 * on dev_nterm above, so the size of this
 				 * structure will vary.
