@@ -125,6 +125,7 @@ int extSpecialPerimFunc();
 
 void extFindDuplicateLabels();
 void extOutputTrans();
+void extOutputParameters();
 void extTransOutTerminal();
 void extTransBad();
 
@@ -217,6 +218,10 @@ extBasic(def, outFile)
 	extFindCoupling(def, &extCoupleHash, (Rect *) NULL);
     }
 
+    /* Output device parameters for any subcircuit devices */
+    if (!SigInterruptPending)
+	extOutputParameters(def, transList, outFile);
+
     /* Output each node, along with its resistance and capacitance to GND */
     if (!SigInterruptPending)
 	extOutputNodes(nodeList, outFile);
@@ -695,7 +700,7 @@ extComputeCapLW(rlengthptr, rwidthptr)
  * first converted to Manhattan geometry.
  *
  * Results:
- *	None
+ *	None.
  *
  * Side Effects:
  *	Puts effective length and width into the pointers
@@ -704,16 +709,24 @@ extComputeCapLW(rlengthptr, rwidthptr)
  */
 
 void
-extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
+extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
    int *rlengthptr, *rwidthptr;
    int numregions;
+   float chop;
 {
     int i, j, p, jmax;
     LinkedBoundary *lb, *lb2;
-    int oppdir, length, loclength, testlen, width, locwidth, testwid;
+    int oppdir, length, loclength, testlen, width;
+    int locwidth, testwid, cornerw;
     int segp, segn, segc, sege;
     bool isComplex = FALSE;
 
+    /* First, check for MOScap-connected transistors.  In such
+     * cases, one or more extSpecialBounds[] is NULL.  Try to
+     * separate the existing extSpecialBounds[] vectors into 
+     * independent (non-connecting) vectors.
+     */
+
     /* For each segment in the primary list, find the closest
      * segment in the other list which lies on the opposite
      * side of the gate area.  Calculate the length, and check
@@ -876,6 +889,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 	/* extension are ignored.					*/
 
 	locwidth = 0;
+	cornerw = 0;
         for (i = 0; i < numregions; i++)
 	{
 	    if (i == p) continue;
@@ -900,14 +914,14 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 			sege = MAX(segp, segn);
 			testwid = segc - sege;
 			if (testwid > loclength) testwid = loclength;
-			if (testwid > 0) locwidth += testwid;
+			if (testwid > 0) cornerw += testwid;
 			
 			/* corner extend bottom */
 			segc = MIN(lb2->r.r_ybot, lb->r.r_ybot);
 			sege = MIN(segp, segn);
 			testwid = sege - segc;
 			if (testwid > loclength) testwid = loclength;
-			if (testwid > 0) locwidth += testwid;
+			if (testwid > 0) cornerw += testwid;
 		    }
 		    else if (((lb->dir == BD_TOP) &&
 			(lb->r.r_ytop - lb2->r.r_ytop == loclength)) ||
@@ -926,14 +940,14 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 			sege = MAX(segp, segn);
 			testwid = segc - sege;
 			if (testwid > loclength) testwid = loclength;
-			if (testwid > 0) locwidth += testwid;
+			if (testwid > 0) cornerw += testwid;
 			
 			/* corner extend left */
 			segc = MIN(lb2->r.r_xbot, lb->r.r_xbot);
 			sege = MIN(segp, segn);
 			testwid = sege - segc;
 			if (testwid > loclength) testwid = loclength;
-			if (testwid > 0) locwidth += testwid;
+			if (testwid > 0) cornerw += testwid;
 		    }
 		}
 	    }
@@ -942,6 +956,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 	    fprintf(stderr, "   segment width = %d\n", width); */
 
 	/* Width scaling for transistor sections with different lengths */
+	locwidth += (int)(0.5 + ((float)cornerw * chop));
 	if (loclength != length)
 	{
 	    locwidth *= length;
@@ -988,6 +1003,9 @@ extSeparateBounds(nterm)
     LinkedBoundary *lb, *lbcomp, *lblast, *lbnext;
     bool found;
 
+    /* Avoid crash condition on a badly-defined extract definition */
+    if ((nterm < 0) || (extSpecialBounds[0] == NULL)) return;
+
     if (extSpecialBounds[nterm] == NULL)
     {
 	/* Put first record into the unused terminal entry */
@@ -1033,6 +1051,57 @@ extSeparateBounds(nterm)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extOutputParameters --
+ *
+ * Scan through the TransRegion in the supplied list, and collect a mask of
+ * all transistor types used in the layout.  Then for each transistor type,
+ * find if it belongs to a "subcircuit" or "rsubcircuit" definition.  If it
+ * does, output a record containing the list of parameter names used by that
+ * subcircuit.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Possibly writes to outFile.  The purpose of this scan is not to have
+ *	to write out shared parameter information for every individual device.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+extOutputParameters(def, transList, outFile)
+    CellDef *def;		/* Cell being extracted */
+    TransRegion *transList;	/* Transistor regions built up in first pass */
+    FILE *outFile;		/* Output file */
+{
+    ParamList *plist;
+    TransRegion *reg;
+    TileType t;
+    TileTypeBitMask tmask;
+
+    TTMaskZero(&tmask);
+    for (reg = transList; reg && !SigInterruptPending; reg = reg->treg_next)
+	TTMaskSetType(&tmask, reg->treg_type);
+
+    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+    {
+	if (TTMaskHasType(&tmask, t))
+	{
+	    plist = ExtCurStyle->exts_deviceParams[t];
+	    if (plist != (ParamList *)NULL)
+	    {
+		fprintf(outFile, "parameters %s", ExtCurStyle->exts_transName[t]);
+		for (; plist != NULL; plist = plist->pl_next)
+		    fprintf(outFile, " %c=%s", plist->pl_param, plist->pl_name);
+		fprintf(outFile, "\n");
+	    }
+	}
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1076,10 +1145,11 @@ extOutputTrans(def, transList, outFile)
     FindRegion arg;
     LabelList *ll;
     TileType t;
-    int nsd, length, width, n, i, ntiles;
+    int nsd, length, width, n, i, ntiles, corners;
     double dres, dcap;
     char mesg[256];
     bool isAnnular, hasModel;
+    ParamList *chkParam;
 
     for (reg = transList; reg && !SigInterruptPending; reg = reg->treg_next)
     {
@@ -1103,13 +1173,18 @@ extOutputTrans(def, transList, outFile)
 	t = reg->treg_type;
 	arg.fra_pNum = DBPlane(t);
 
-	    /* Mark with reg and process each perimeter segment */
+	/* Set all terminals to NULL to guard against	*/
+	/* asymmetric devices missing a terminal.	*/
+	nsd = ExtCurStyle->exts_transSDCount[t];
+	for (i = 0; i < nsd; i++) extTransRec.tr_termnode[i] = NULL;
+
+	/* Mark with reg and process each perimeter segment */
 	arg.fra_uninit = (ClientData) extTransRec.tr_gatenode;
 	arg.fra_region = (Region *) reg;
 	arg.fra_each = extTransTileFunc;
 	ntiles = ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
-	    /* Re-mark with extTransRec.tr_gatenode */
+	/* Re-mark with extTransRec.tr_gatenode */
 	arg.fra_uninit = (ClientData) reg;
 	arg.fra_region = (Region *) extTransRec.tr_gatenode;
 	arg.fra_each = (int (*)()) NULL;
@@ -1121,7 +1196,6 @@ extOutputTrans(def, transList, outFile)
 	 * issue a warning message and make believe the missing
 	 * terminals are the same as the last terminal we do have.
 	 */
-	nsd = ExtCurStyle->exts_transSDCount[t];
 	if (extTransRec.tr_nterm < nsd)
 	{
 	    int missing = nsd - extTransRec.tr_nterm;
@@ -1177,9 +1251,11 @@ extOutputTrans(def, transList, outFile)
 	if (ExtCurStyle->exts_deviceClass[t] != DEV_FET)
 	    fprintf(outFile, "device ");
 
-	fprintf(outFile, "%s %s %d %d %d %d",
-		extDevTable[ExtCurStyle->exts_deviceClass[t]],
-		ExtCurStyle->exts_transName[t],
+	fprintf(outFile, "%s %s",
+			extDevTable[ExtCurStyle->exts_deviceClass[t]],
+			ExtCurStyle->exts_transName[t]);
+
+	fprintf(outFile, " %d %d %d %d",
 		reg->treg_ll.p_x, reg->treg_ll.p_y, 
 		reg->treg_ll.p_x + 1, reg->treg_ll.p_y + 1);
 
@@ -1206,7 +1282,6 @@ extOutputTrans(def, transList, outFile)
 	    case DEV_MOSFET:
 	    case DEV_BJT:
 	    case DEV_SUBCKT:
-	    case DEV_RSUBCKT:
 		length = extTransRec.tr_gatelen / 2;	/* (default) */
 		width = 0;
 		isAnnular = FALSE;
@@ -1216,9 +1291,10 @@ extOutputTrans(def, transList, outFile)
 		/* for transistors w/connected S-D is to count over	*/
 		/* non-NULL termnodes, not non-zero termlens.		*/
 
-		for (n = 0; n < extTransRec.tr_nterm &&
-				extTransRec.tr_termnode[n] != NULL; n++)
+		for (n = 0; n < extTransRec.tr_nterm; n++)
 		{
+		    if (extTransRec.tr_termnode[n] == NULL) continue;
+
 		    width += extTransRec.tr_termlen[n];
 
 		    /* Mark annular transistors as requiring extra processing */
@@ -1241,14 +1317,14 @@ extOutputTrans(def, transList, outFile)
 		/* device.						*/
 		/*------------------------------------------------------*/
 
-		/* NOTE: tr_termvector for non-Manhattan bent devices	*/
-		/* is the same as for a single rectangle.		*/
-
-#ifdef NONMANHATTAN
-		if (1)
-#else
-		if (isAnnular)
-#endif
+		if (n == 0)
+		{
+		    if (ExtDoWarn)
+			extTransBad(def, reg->treg_tile,
+				"Could not determine device boundary");
+		    length = width = 0;
+		}
+		else
 		{
 		    LinkedBoundary *lb;
 
@@ -1264,7 +1340,9 @@ extOutputTrans(def, transList, outFile)
 		    arg.fra_each = extAnnularTileFunc;
 		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
-		    extComputeEffectiveLW(&length, &width, n);
+		    extSeparateBounds(n - 1);	/* Handle MOScaps (if necessary) */
+		    extComputeEffectiveLW(&length, &width, n,
+				ExtCurStyle->exts_cornerChop[t]);
 
 		    /* Free the lists */
 
@@ -1280,29 +1358,82 @@ extOutputTrans(def, transList, outFile)
 		    arg.fra_region = (Region *) extTransRec.tr_gatenode;
 		    arg.fra_each = (int (*)()) NULL;
 		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
+
 		}
 
-		fprintf(outFile, " %d %d \"%s\"", length, width,
-			(subsName == NULL) ? "None" : subsName);
+		if (ExtCurStyle->exts_deviceClass[t] == DEV_SUBCKT)
+		{
+		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
+				!= NULL; chkParam = chkParam->pl_next)
+		    {
+			switch(tolower(chkParam->pl_param))
+			{
+			    case 'a':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+						reg->treg_area);
+				break;
+			    case 'p':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+						extTransRec.tr_perim);
+				break;
+			    case 'l':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+						length);
+				break;
+			    case 'w':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+						width);
+				break;
+			    case 'c':
+				fprintf(outFile, " %c=%g", chkParam->pl_param,
+					(ExtCurStyle->exts_transGateCap[t]
+					* reg->treg_area) +
+					(ExtCurStyle->exts_transSDCap[t]
+					* extTransRec.tr_perim));
+				break;
+			    case 's':
+			    case 'x':
+			    case 'y':
+				/* Do nothing;  these values are standard output */
+				break;
+			    default:
+				fprintf(outFile, " %c=", chkParam->pl_param);
+				break;
+			}
+		    }
+
+		}
+		else  /* DEV_MOSFET, DEV_BJT */
+		{
+		    fprintf(outFile, " %d %d", length, width);
+		}
+
+		fprintf(outFile, " \"%s\"", (subsName == NULL) ?
+					"None" : subsName);
 		break;
 
-	    case DEV_DIODE:	/* Do nothing */
+	    case DEV_DIODE:	/* Only handle the optional substrate node */
+		if (subsName != NULL)
+		    fprintf(outFile, " \"%s\"", subsName);
 		break;
 
 	    case DEV_RES:
+	    case DEV_RSUBCKT:
 		hasModel = strcmp(ExtCurStyle->exts_transName[t], "None");
 		length = extTransRec.tr_perim;
 	
 		/* Boundary perimeter scan for resistors with more than */
 		/* one tile.						*/
 
-		if (ntiles > 1)
+		for (n = 0; n < extTransRec.tr_nterm &&
+				extTransRec.tr_termnode[n] != NULL; n++);
+
+		if (n == 0)
+		    width = length = 0;
+		else if (ntiles > 1)
 		{
 		    LinkedBoundary *lb;
 
-		    for (n = 0; n < extTransRec.tr_nterm &&
-				extTransRec.tr_termnode[n] != NULL; n++);
-
 		    extSpecialBounds = (LinkedBoundary **)mallocMagic(n *
 				sizeof(LinkedBoundary *));
 
@@ -1318,15 +1449,15 @@ extOutputTrans(def, transList, outFile)
 		    if (extSpecialBounds[0] != NULL)
 		    {
 			extSeparateBounds(n - 1);
-			extComputeEffectiveLW(&width, &length, n);
+			extComputeEffectiveLW(&width, &length, n, 
+				ExtCurStyle->exts_cornerChop[t]);
 		    }
 		    else
 		    {
 			if (ExtDoWarn)
 			    extTransBad(def, reg->treg_tile,
 					"Could not determine resistor boundary");
-			width = 0;
-			length = 0;
+			length = width = 0;
 		    }
 
 		    /* Free the lists */
@@ -1360,8 +1491,7 @@ extOutputTrans(def, transList, outFile)
 		}
 		if (width)
 		{
-		    if (!hasModel)
-			dres = ExtCurStyle->exts_sheetResist[t] * (double)length /
+		    dres = ExtCurStyle->exts_sheetResist[t] * (double)length /
 				(double)width;
 		    if (ExtDoWarn && (n > 2))
 		    {
@@ -1381,8 +1511,59 @@ extOutputTrans(def, transList, outFile)
 					"Resistor has zero width");
 		}
 
-		if (hasModel)	/* semiconductor resistor */
+		if (ExtCurStyle->exts_deviceClass[t] == DEV_RSUBCKT)
+		{
+		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
+				!= NULL; chkParam = chkParam->pl_next)
+		    {
+			switch(tolower(chkParam->pl_param))
+			{
+			    case 'a':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+					reg->treg_area);
+				break;
+			    case 'p':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+					extTransRec.tr_perim);
+				break;
+			    case 'l':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+					length);
+				break;
+			    case 'w':
+				fprintf(outFile, " %c=%d", chkParam->pl_param,
+					width);
+				break;
+			    case 'r':
+				fprintf(outFile, " %c=%g", chkParam->pl_param,
+					dres / 1000.0);
+				break;
+			    case 'c':
+				fprintf(outFile, " %c=%g", chkParam->pl_param,
+					(ExtCurStyle->exts_transGateCap[t]
+					* reg->treg_area) +
+					(ExtCurStyle->exts_transSDCap[t]
+					* extTransRec.tr_perim));
+				break;
+			    case 's':
+			    case 'x':
+			    case 'y':
+				/* Do nothing;  these values are standard output */
+				break;
+			    default:
+				fprintf(outFile, " %c=", chkParam->pl_param);
+				break;
+			}
+		    }
+		    fprintf(outFile, " \"%s\"", (subsName == NULL) ?
+				"None" : subsName);
+		}
+		else if (hasModel)	/* SPICE semiconductor resistor */
+		{
 		    fprintf(outFile, " %d %d", length, width);
+		    if (subsName != NULL)
+			fprintf(outFile, " \"%s\"", subsName);
+		}
 		else		/* regular resistor */
 		    fprintf(outFile, " %g", dres / 1000.0); /* mOhms -> Ohms */
 		break;
@@ -1398,21 +1579,27 @@ extOutputTrans(def, transList, outFile)
 		    /* multiple terminals (see below); treat them in	*/
 		    /* the original naive manner.			*/
 
-//		    if ((ntiles == 1) || (n > 1))
-		    if (0)
+		    if (n == 0)
 		    {
 			width = 0;
-			for (n = 0; extTransRec.tr_termlen[n] != 0; n++)
-			    width += extTransRec.tr_termlen[n];
-
-			if (!width && ExtDoWarn)
-			    extTransBad(def, reg->treg_tile,
-					"Capacitor has zero width");
-			else width >>= 2;	/* total perimeter / 4 */
-
-			fprintf(outFile, " %d %d", reg->treg_area / width,
-				width);
+			extTransBad(def, reg->treg_tile,
+						"Capacitor has zero size");
+			fprintf(outFile, " 0 0");
 		    }
+//		    else if ((ntiles == 1) || (n > 1))
+//		    {
+//			width = 0;
+//			for (n = 0; extTransRec.tr_termlen[n] != 0; n++)
+//			    width += extTransRec.tr_termlen[n];
+//
+//			if (!width && ExtDoWarn)
+//			    extTransBad(def, reg->treg_tile,
+//					"Capacitor has zero width");
+//			else width >>= 2;	/* total perimeter / 4 */
+//
+//			fprintf(outFile, " %d %d", reg->treg_area / width,
+//				width);
+//		    }
 		    else
 		    {
 			/* Special handling of multiple-tile areas.	*/
@@ -1463,6 +1650,8 @@ extOutputTrans(def, transList, outFile)
 			arg.fra_each = (int (*)()) NULL;
 			(void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 		    }
+		    if (subsName != NULL)
+			fprintf(outFile, " \"%s\"", subsName);
 		}
 		else
 		{
@@ -1480,7 +1669,12 @@ extOutputTrans(def, transList, outFile)
 	extTransOutTerminal((LabRegion *) node, ll, LL_GATEATTR,
 			extTransRec.tr_gatelen, outFile);
 
-	ExtSortTerminals(&extTransRec, ll); 
+	/* Sort source and drain terminals by position, unless the	*/
+	/* device is asymmetric, in which case source and drain do not	*/
+	/* permute, and the terminal order is fixed.			*/
+
+	if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[t][1], TT_SPACE))
+	    ExtSortTerminals(&extTransRec, ll); 
 
 	/* each non-gate terminal */
 	for (nsd = 0; nsd < extTransRec.tr_nterm; nsd++)
@@ -1581,6 +1775,7 @@ extTransTileFunc(tile, pNum)
     if (IsSplit(tile))
     {
         loctype = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+	return (0); 	/* Hack alert!  We must properly handle diagonals! */
     }
     else
 #endif
@@ -1599,93 +1794,113 @@ extTransPerimFunc(bp)
     TileType tinside = TiGetType(bp->b_inside);
     TileType toutside = TiGetType(bp->b_outside);
     NodeRegion *diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
-    int len = BoundaryLength(bp);
+    int i, len = BoundaryLength(bp);
     int thisterm;
     LabelList *ll;
     Label *lab;
     Rect r;
-
-    if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside], toutside))
+    bool SDterm = FALSE;
+ 
+    for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], TT_SPACE);
+		i++)
     {
-	/*
-	 * It's a diffusion terminal (source or drain).
-	 * See if the node is already in our table; add it
-	 * if it wasn't already there.
-	 */
-	for (thisterm = 0; thisterm < extTransRec.tr_nterm; thisterm++)
-	    if (extTransRec.tr_termnode[thisterm] == diffNode)
-		break;
-	if (thisterm >= extTransRec.tr_nterm)
+	if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], toutside))
 	{
-	    extTransRec.tr_nterm++;
-	    extTransRec.tr_termnode[thisterm] = diffNode;
-	    extTransRec.tr_termlen[thisterm] = 0;
-	    extTransRec.tr_termvector[thisterm].p_x = 0;
-	    extTransRec.tr_termvector[thisterm].p_y = 0;
-	    extTransRec.tr_termpos[thisterm].pnum = DBPlane(toutside);
-	    extTransRec.tr_termpos[thisterm].pt = bp->b_outside->ti_ll;
-
-	    /* Find the total area of this terminal */
-	}
-	else			/* update the region tile position */
-	{
-	    TermTilePos  *pos = &(extTransRec.tr_termpos[thisterm]);
-	    Tile         *otile = bp->b_outside;
-
-	    if( DBPlane(TiGetType(otile)) < pos->pnum )
+	    /*
+	     * It's a diffusion terminal (source or drain).  See if the node is
+	     * already in our table; add it if it wasn't already there.
+	     * Asymmetric devices must have terminals in order.
+	     */
+	    if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][1], TT_SPACE))
 	    {
-		pos->pnum = DBPlane(TiGetType(otile));
-		pos->pt = otile->ti_ll;
+		for (thisterm = 0; thisterm < extTransRec.tr_nterm; thisterm++)
+		    if (extTransRec.tr_termnode[thisterm] == diffNode)
+			break;
 	    }
-	    else if( DBPlane(TiGetType(otile)) == pos->pnum )
+	    else
+		thisterm = i;
+
+	    if (extTransRec.tr_termnode[thisterm] == NULL)
 	    {
-		if( LEFT(otile) < pos->pt.p_x )
+		extTransRec.tr_nterm++;
+		extTransRec.tr_termnode[thisterm] = diffNode;
+		extTransRec.tr_termlen[thisterm] = 0;
+		extTransRec.tr_termvector[thisterm].p_x = 0;
+		extTransRec.tr_termvector[thisterm].p_y = 0;
+		extTransRec.tr_termpos[thisterm].pnum = DBPlane(toutside);
+		extTransRec.tr_termpos[thisterm].pt = bp->b_outside->ti_ll;
+
+		/* Find the total area of this terminal */
+	    }
+	    else if (extTransRec.tr_termnode[thisterm] == diffNode)
+	    {
+		TermTilePos  *pos = &(extTransRec.tr_termpos[thisterm]);
+		Tile         *otile = bp->b_outside;
+
+		/* update the region tile position */
+
+		if( DBPlane(TiGetType(otile)) < pos->pnum )
+		{
+		    pos->pnum = DBPlane(TiGetType(otile));
 		    pos->pt = otile->ti_ll;
-		else if( LEFT(otile) == pos->pt.p_x && 
-		  BOTTOM(otile) < pos->pt.p_y )
-		    pos->pt.p_y = BOTTOM(otile);
+		}
+		else if( DBPlane(TiGetType(otile)) == pos->pnum )
+		{
+		    if( LEFT(otile) < pos->pt.p_x )
+			pos->pt = otile->ti_ll;
+		    else if( LEFT(otile) == pos->pt.p_x && 
+				BOTTOM(otile) < pos->pt.p_y )
+			pos->pt.p_y = BOTTOM(otile);
+		}
 	    }
-	}
-
-	/* Add the length to this terminal's perimeter */
-	extTransRec.tr_termlen[thisterm] += len;
-
-	/* Update the boundary traversal vector */
-	switch(bp->b_direction) {
-	    case BD_LEFT:
-		extTransRec.tr_termvector[thisterm].p_y += len;
-		break;
-	    case BD_TOP:
-		extTransRec.tr_termvector[thisterm].p_x += len;
-		break;
-	    case BD_RIGHT:
-		extTransRec.tr_termvector[thisterm].p_y -= len;
-		break;
-	    case BD_BOTTOM:
-		extTransRec.tr_termvector[thisterm].p_x -= len;
-		break;
-	}
-
-	/*
-	 * Mark this attribute as belonging to this transistor
-	 * if it is either:
-	 *	(1) a terminal attribute whose LL corner touches bp->b_segment,
-	 *   or	(2) a gate attribute that lies inside bp->b_inside.
-	 */
-	for (ll = extTransRec.tr_gatenode->nreg_labels; ll; ll = ll->ll_next)
-	{
-	    /* Skip if already marked */
-	    if (ll->ll_attr != LL_NOATTR)
-		continue;
-	    lab = ll->ll_label;
-	    if (GEO_ENCLOSE(&lab->lab_rect.r_ll, &bp->b_segment)
-		    && extLabType(lab->lab_text, LABTYPE_TERMATTR))
+	    else
 	    {
-		ll->ll_attr = thisterm;
+		TxError("Error:  Asymmetric device with multiple terminals!\n");
 	    }
+
+	    /* Add the length to this terminal's perimeter */
+	    extTransRec.tr_termlen[thisterm] += len;
+
+	    /* Update the boundary traversal vector */
+	    switch(bp->b_direction) {
+		case BD_LEFT:
+		    extTransRec.tr_termvector[thisterm].p_y += len;
+		    break;
+		case BD_TOP:
+		    extTransRec.tr_termvector[thisterm].p_x += len;
+		    break;
+		case BD_RIGHT:
+		    extTransRec.tr_termvector[thisterm].p_y -= len;
+		    break;
+		case BD_BOTTOM:
+		    extTransRec.tr_termvector[thisterm].p_x -= len;
+		    break;
+	    }
+
+	    /*
+	     * Mark this attribute as belonging to this transistor
+	     * if it is either:
+	     *	(1) a terminal attribute whose LL corner touches bp->b_segment,
+	     *   or	(2) a gate attribute that lies inside bp->b_inside.
+	     */
+	    for (ll = extTransRec.tr_gatenode->nreg_labels; ll; ll = ll->ll_next)
+	    {
+		/* Skip if already marked */
+		if (ll->ll_attr != LL_NOATTR)
+		    continue;
+		lab = ll->ll_label;
+		if (GEO_ENCLOSE(&lab->lab_rect.r_ll, &bp->b_segment)
+			&& extLabType(lab->lab_text, LABTYPE_TERMATTR))
+		{
+		    ll->ll_attr = thisterm;
+		}
+	    }
+	    SDterm = TRUE;
+	    break;
 	}
     }
-    else if (extConnectsTo(tinside, toutside, ExtCurStyle->exts_nodeConn))
+
+    if (!SDterm && extConnectsTo(tinside, toutside, ExtCurStyle->exts_nodeConn))
     {
 	/* Not in a terminal, but are in something that connects to gate */
 	extTransRec.tr_gatelen += len;
@@ -1795,7 +2010,7 @@ extResistorTileFunc(tile, pNum)
     loctype = TiGetTypeExact(tile);
 
     mask = ExtCurStyle->exts_transConn[loctype];
-    TTMaskSetMask(&mask, &ExtCurStyle->exts_transSDTypes[loctype]);
+    TTMaskSetMask(&mask, &ExtCurStyle->exts_transSDTypes[loctype][0]);
     TTMaskCom(&mask);
 
     extEnumTilePerim(tile, mask, pNum, extSpecialPerimFunc, (ClientData)FALSE);
@@ -1816,9 +2031,9 @@ extSpecialPerimFunc(bp, sense)
 {
     TileType tinside, toutside;
     NodeRegion *diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
-    int thisterm;
-    int extended;
+    int thisterm, extended, i;
     LinkedBoundary *newBound, *lb, *lastlb;
+    bool needSurvey;
 
 #ifdef NONMANHATTAN
     /* Note that extEnumTilePerim() assumes for the non-Manhattan case	*/
@@ -1860,7 +2075,19 @@ extSpecialPerimFunc(bp, sense)
     toutside = TiGetType(bp->b_outside);
 #endif
 
-    if (!sense || TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside], toutside))
+    /* Check all terminal classes for a matching type */
+    needSurvey = FALSE;
+    for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], TT_SPACE);
+		i++)
+    {
+	if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], toutside))
+	{
+	    needSurvey = TRUE;
+	    break;
+	}
+    }
+
+    if (!sense || needSurvey)
     {
 	/*
 	 * Since we're repeating the search, all terminals should be there.
