@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtHard.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtHard.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -45,7 +45,73 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 void extHardFreeAll();
 bool extHardGenerateLabel();
 bool extHardSetLabel();
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extLabFirst --
+ * extLabEach --
+ *
+ * Filter functions passed to ExtFindRegions when tracing out extended
+ * label regions as part of hierarchical circuit extraction.  We use the
+ * TransRegion record because it allows us to save a pointer to a tile
+ * along with the usual LabRegion data.  Since treg_area is not needed,
+ * we use it to store the plane of the tile.
+ *
+ * Results:
+ *	extLabFirst returns a pointer to a new TransRegion.
+ *	extLabEach returns NULL.
+ *
+ * Side effects:
+ *	Memory is allocated by extLabFirst.
+ *	We cons the newly allocated region onto the front of the existing
+ *	region list.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+Region *
+extLabFirst(tile, arg)
+    Tile *tile;
+    FindRegion *arg;
+{
+    TransRegion *reg;
+    reg = (TransRegion *) mallocMagic((unsigned) (sizeof (TransRegion)));
+    reg->treg_next = (TransRegion *) NULL;
+    reg->treg_labels = (LabelList *) NULL;
+    reg->treg_pnum = DBNumPlanes;
+    reg->treg_area = DBNumPlanes;
+    reg->treg_tile = tile;
+
+    /* Prepend it to the region list */
+    reg->treg_next = (TransRegion *) arg->fra_region;
+    arg->fra_region = (Region *) reg;
+    return ((Region *) reg);
+}
+
+    /*ARGSUSED*/
+int
+extLabEach(tile, pNum, arg)
+    Tile *tile;
+    int pNum;
+    FindRegion *arg;
+{
+    TransRegion *reg = (TransRegion *) arg->fra_region;
+
+#ifdef NONMANHATTAN
+    /* Avoid setting the region's tile pointer to a split tile if we can */
+    if (IsSplit(reg->treg_tile) && !IsSplit(tile))
+    {
+	reg->treg_tile = tile;
+	reg->treg_area = pNum;
+    }
+#endif
+
+    if (reg->treg_area == DBNumPlanes) reg->treg_area = pNum;
+    extSetNodeNum((LabRegion *)reg, pNum, tile);
+    return (0);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -118,14 +184,15 @@ extHardProc(scx, arg)
     if (arg->hw_prefix || !isTopLevel)
     {
 	arg->hw_tpath.tp_next =
-	    DBPrintUseId(scx, savenext, arg->hw_tpath.tp_last - savenext);
+	    DBPrintUseId(scx, savenext, arg->hw_tpath.tp_last - savenext,
+			FALSE);
 	*arg->hw_tpath.tp_next++ = '/';
 	*arg->hw_tpath.tp_next = '\0';
     }
 
     /*
      * We use a TransRegion because it holds both a LabelList
-     * and a tile pointer, and call extTransEach to make sure
+     * and a tile pointer, and call extLabEach to make sure
      * that treg_pnum and treg_ll are kept up-to-date (so we
      * can generate a node label if none is found).
      *
@@ -135,7 +202,7 @@ extHardProc(scx, arg)
      */
     labRegList = (TransRegion *) ExtFindRegions(def, &scx->scx_area,
 		    &arg->hw_mask, ExtCurStyle->exts_nodeConn, extUnInit,
-		    extTransFirst, extTransEach);
+		    extLabFirst, extLabEach);
     if (labRegList)
     {
 	/*
@@ -276,7 +343,7 @@ extHardSetLabel(scx, reg, arg)
     text = oldlab->lab_text;
 
     /* Don't care, really, which orientation the label has */
-    newlab->lab_pos = GEO_NORTH;
+    newlab->lab_just = GEO_NORTH;
 
     /* Construct the text of the new label */
     dstp = newlab->lab_text;
@@ -349,7 +416,7 @@ extHardGenerateLabel(scx, reg, arg)
     newlab->lab_type = TiGetType(reg->treg_tile);
 
     /* Don't care, really, which orientation the label has */
-    newlab->lab_pos = GEO_NORTH;
+    newlab->lab_just = GEO_NORTH;
 
     /* Construct the text of the new label */
     dstp = newlab->lab_text;
@@ -403,7 +470,7 @@ extHardFreeAll(def, tReg)
     {
 	/* Reset all ti_client fields to extUnInit */
 	arg.fra_uninit = (ClientData) reg;
-	arg.fra_pNum = DBPlane(TiGetType(reg->treg_tile));
+	arg.fra_pNum = reg->treg_area;
 	(void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
 	/* Free all LabelLists and then the region */
