@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtLength.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtLength.c,v 1.2 2008/06/01 18:37:42 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -444,8 +444,8 @@ extLengthLabels(tile, rootUse)
     tpath.tp_last = &name[sizeof name - 2];
 
     TTMaskSetOnlyType(&mask, TiGetType(tile));
-    (void) DBTreeSrLabels(&scx, &mask, 0, &tpath, extLengthLabelsFunc,
-	    (ClientData) NULL);
+    (void) DBTreeSrLabels(&scx, &mask, 0, &tpath, TF_LABEL_ATTACH,
+		extLengthLabelsFunc, (ClientData) NULL);
 
     return (0);
 }
