@@ -109,7 +109,7 @@ static keydesc keyTable[] = {
     "defaultsidewall",	DEFAULTSIDEWALL, 4,	4,
 "types plane capacitance",
 
-    "device",		DEVICE,		4,	9,
+    "device",		DEVICE,		4,	10,
 "device dev-type types options...",
 
     "fet",		FET,		8,	9,
@@ -135,7 +135,7 @@ static keydesc keyTable[] = {
     "noplaneordering",	NOPLANEORDER,	1,	1,
 "(no arguments needed)",
 
-    "resist",		RESIST,		3,	3,
+    "resist",		RESIST,		3,	4,
 "types resistance",
 
     "rscale",		RSCALE,		2,	2,
@@ -168,31 +168,34 @@ static keydesc keyTable[] = {
 
 /*
  * Table used for parsing the "device" keyword types
+ *
+ * (Note: "10" for max types in subcircuit is arbitrary---the parser
+ * ignores max types for DEV_SUBCKT).
  */
 
 /* types are enumerated in extract.h */
 
 static keydesc devTable[] = {
-    "mosfet",		DEV_MOSFET,		5,	9,
-"name gate-types src-drn-types sub-types|None sub-node [gscap gccap]",
+    "mosfet",		DEV_MOSFET,		5,	10,
+"name gate-types src-types [drn-types] sub-types|None sub-node [gscap gccap]",
 
     "bjt",		DEV_BJT,		5,	5,
 "name base-types emitter-types collector-types",
 
-    "capacitor",	DEV_CAP,		5,	6,
-"name top-types bottom-types [perimcap] areacap",
+    "capacitor",	DEV_CAP,		5,	8,
+"name top-types bottom-types [sub-types|None sub-node] [perimcap] areacap",
 
-    "resistor",		DEV_RES,		4,	4,
-"name|None res-types terminal-types",
+    "resistor",		DEV_RES,		4,	6,
+"name|None res-types terminal-types [sub-types|None sub-node]",
 
-    "diode",		DEV_DIODE,		4,	4,
-"name pos-types neg-types",
+    "diode",		DEV_DIODE,		4,	6,
+"name pos-types neg-types [sub-types|None sub-node]",
 
-    "subcircuit",	DEV_SUBCKT,		3,	5,
-"name dev-types [term-types [sub-types]]",
+    "subcircuit",	DEV_SUBCKT,		3,	11,
+"name dev-types [N] [term1-types ... termN-types [sub-types|None sub-node]] [options]",
 
-    "rsubcircuit",	DEV_RSUBCKT,		4,	4,
-"name dev-types terminal-types",
+    "rsubcircuit",	DEV_RSUBCKT,		4,	7,
+"name dev-types terminal-types [sub-types|None sub-node] [options]",
 
     0
 };
@@ -269,7 +272,7 @@ bool
 ExtGetDevInfo(idx, devnameptr, sd_rclassptr, sub_rclassptr, subnameptr)
     int idx;
     char **devnameptr;
-    short *sd_rclassptr;
+    short *sd_rclassptr;	/* First SD type only---needs to be updated! */
     short *sub_rclassptr;
     char **subnameptr;
 {
@@ -306,12 +309,12 @@ ExtGetDevInfo(idx, devnameptr, sd_rclassptr, sub_rclassptr, subnameptr)
     *devnameptr = locdname;
     *subnameptr = ExtCurStyle->exts_transSubstrateName[t];
 
-    tmask = ExtCurStyle->exts_transSDTypes[t];
+    tmask = ExtCurStyle->exts_transSDTypes[t][0];
     *sd_rclassptr = (short)(-1);	/* NO_RESCLASS */
 
     for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
     {
-        rmask = ExtCurStyle->exts_typesByResistClass[n];
+	rmask = ExtCurStyle->exts_typesByResistClass[n];
 	if (TTMaskAndMask(&rmask, &tmask))
 	{
 	    *sd_rclassptr = (short)n;
@@ -525,6 +528,7 @@ extTechStyleAlloc()
     {
 	style->exts_transSubstrateName[r] = (char *) NULL;
 	style->exts_transName[r] = (char *) NULL;
+	style->exts_deviceParams[r] = (ParamList *) NULL;
 	style->exts_deviceClass[r] = (char) 0;
 	style->exts_transResist[r].ht_table = (HashEntry **) NULL;
     }
@@ -569,6 +573,7 @@ extTechStyleInit(style)
 	style->exts_allConn[r] = DBAllTypeBits;
 
 	style->exts_sheetResist[r] = 0;
+	style->exts_cornerChop[r] = 1.0;
 	style->exts_viaResist[r] = 0;
 	style->exts_height[r] = 0.0;
 	style->exts_thick[r] = 0.0;
@@ -592,8 +597,10 @@ extTechStyleInit(style)
 	}
 
 	TTMaskZero(&style->exts_perimCapMask[r]);
-	TTMaskZero(&style->exts_transSDTypes[r]);
 	TTMaskZero(&style->exts_subsTransistorTypes[r]);
+	if (style->exts_transSDTypes[r] != NULL)
+	    freeMagic(style->exts_transSDTypes[r]);
+	style->exts_transSDTypes[r] = NULL;
 	style->exts_transSDCount[r] = 0;
 	style->exts_transGateCap[r] = (CapValue) 0;
 	style->exts_transSDCap[r] = (CapValue) 0;
@@ -607,6 +614,22 @@ extTechStyleInit(style)
 	    freeMagic(style->exts_transName[r]);
 	    style->exts_transName[r] = (char *) NULL;
 	}
+	while (style->exts_deviceParams[r] != (ParamList *) NULL)
+	{
+	    /* Parameter lists are shared.  Only free the last one! */
+
+	    if (style->exts_deviceParams[r]->pl_count > 1)
+	    {
+		style->exts_deviceParams[r]->pl_count--;
+		style->exts_deviceParams[r] = (ParamList *)NULL;
+	    }
+	    else
+	    {
+		freeMagic(style->exts_deviceParams[r]->pl_name);
+		freeMagic(style->exts_deviceParams[r]);
+		style->exts_deviceParams[r] = style->exts_deviceParams[r]->pl_next;
+	    }
+	}
 	style->exts_deviceClass[r] = (char)0;
 	if (style->exts_transResist[r].ht_table != (HashEntry **) NULL)
 	    HashKill(&style->exts_transResist[r]);
@@ -615,7 +638,6 @@ extTechStyleInit(style)
     }
 
     style->exts_sideCoupleHalo = 0;
-
     style->exts_stepSize = 100;
     style->exts_unitsPerLambda = 100.0;
     style->exts_resistScale = 1000;
@@ -1079,7 +1101,7 @@ ExtTechSimpleSidewallCap(argv)
 	    ExtCurStyle->exts_sidePlanes |= PlaneNumToMaskBit(DBPlane(s));
 	    TTMaskSetType(&ExtCurStyle->exts_sideTypes[DBPlane(s)], s);
 	    TTMaskSetMask(&ExtCurStyle->exts_sideEdges[s], &types2);
-	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	    for (t = 0; t < DBNumTypes; t++)
 	    {
 		if (!TTMaskHasType(&types2, t))
 		    continue;
@@ -1458,6 +1480,8 @@ ExtTechSimpleSideOverlapCap(argv)
  * ----------------------------------------------------------------------------
  */
 
+#define MAXSD 6
+
     /*ARGSUSED*/
 bool
 ExtTechLine(sectionName, argc, argv)
@@ -1465,17 +1489,20 @@ ExtTechLine(sectionName, argc, argv)
     int argc;
     char *argv[];
 {
-    int n, l, j, size, val, p1, p2, p3, nterm, class;
+    int n, l, i, j, size, val, p1, p2, p3, nterm, iterm, class;
     PlaneMask pshield, pov;
     CapValue capVal, gscap, gccap;
-    TileTypeBitMask types1, types2, near, far, ov, shield, subsTypes;
-    char *subsName, *transName, *cp, *endptr;
+    TileTypeBitMask types1, types2, termtypes[MAXSD];
+    TileTypeBitMask near, far, ov, shield, subsTypes;
+    char *subsName, *transName, *cp, *endptr, *paramName;
     TileType s, t, r, o;
     keydesc *kp, *dv;
     bool isLinear;
     HashEntry *he;
     EdgeCap *cnew;
     ExtKeep *es, *newStyle;
+    ParamList *subcktParams, *newParam;
+    int refcnt;
     bool bad;
 
     if (argc < 1)
@@ -1495,8 +1522,13 @@ ExtTechLine(sectionName, argc, argv)
     }
 
     kp = &keyTable[n];
-    if (argc < kp->k_minargs || argc > kp->k_maxargs)
+    if (argc < kp->k_minargs)
 	goto usage;
+
+    /* Handle maxargs for DEVICE type separately */
+    if ((argc > kp->k_maxargs) && (kp->k_key != DEVICE))
+	goto usage;
+
     else if (argc >= 2) l = strlen(argv[1]);
 
     /* If ExtCurStyle is NULL, this is a first pass, and we should	*/
@@ -1747,7 +1779,7 @@ ExtTechLine(sectionName, argc, argv)
 
 	    /* Original FET format, kept for backwards compatibility */
 
-	    DBTechNoisyNameMask(argv[2], &types2);
+	    DBTechNoisyNameMask(argv[2], &termtypes[0]);
 	    nterm = atoi(argv[3]);
 	    transName = argv[4];
 	    subsName = argv[5];
@@ -1774,7 +1806,10 @@ ExtTechLine(sectionName, argc, argv)
 		if (TTMaskHasType(&types1, t))
 		{
 		    TTMaskSetMask(ExtCurStyle->exts_transConn+t,&types1);
-		    ExtCurStyle->exts_transSDTypes[t] = types2;
+		    ExtCurStyle->exts_transSDTypes[t] = (TileTypeBitMask *)
+				mallocMagic(2 * sizeof(TileTypeBitMask));
+		    ExtCurStyle->exts_transSDTypes[t][0] = termtypes[0];
+		    ExtCurStyle->exts_transSDTypes[t][1] = DBSpaceBits;
 		    ExtCurStyle->exts_transSDCount[t] = nterm;
 		    ExtCurStyle->exts_transSDCap[t] = gscap;
 		    ExtCurStyle->exts_transGateCap[t] = gccap;
@@ -1828,59 +1863,193 @@ ExtTechLine(sectionName, argc, argv)
 	    }
 
 	    dv = &devTable[n];
-	    if ((argc - 1) < dv->k_minargs || (argc - 1) > dv->k_maxargs)
+	    if ((argc - 1) < dv->k_minargs)
 		goto usage;
 
+	    /* No limit on arguments in DEV_SUBCKT! */
+	    /* And. . . check DEV_RSUBCKT later, after parsing parameter names */
+	    switch (dv->k_key)
+	    {
+		case DEV_SUBCKT:
+		case DEV_RSUBCKT:
+		    break;
+		default:
+		    if ((argc - 1) > dv->k_maxargs)
+			goto usage;
+		    break;
+	    }
+
 	    gscap = (CapValue) 0;
 	    gccap = (CapValue) 0;
 	    subsName = NULL;
+	    subcktParams = NULL;
 	    subsTypes = DBZeroTypeBits;
 	    transName = argv[2];
 
 	    switch (dv->k_key)
 	    {
 		case DEV_BJT:
-		    DBTechNoisyNameMask(argv[4], &types2);	/* emitter */
-		    DBTechNoisyNameMask(argv[5], &subsTypes);	/* collector */
+		    DBTechNoisyNameMask(argv[4], &termtypes[0]); /* emitter */
+		    termtypes[1] = DBSpaceBits;
+		    DBTechNoisyNameMask(argv[5], &subsTypes);	 /* collector */
 		    nterm = 1;	/* emitter is the only "terminal type" expected */
 		    break;
 		case DEV_MOSFET:
-		    DBTechNoisyNameMask(argv[4], &types2);	/* source/drain */
-		    if (strcmp(argv[5], "None"))
-			DBTechNoisyNameMask(argv[5], &subsTypes);   /* substrate */
-		    subsName = argv[6];
-		    if (argc > 7) gscap = aToCap(argv[7]);
-		    if (argc > 8) gccap = aToCap(argv[8]);
-		    nterm = 2;
+		    if ((argc > 7) && (!StrIsNumeric(argv[7])))
+		    {
+			/* Asymmetric device with different source and drain types */
+
+			DBTechNoisyNameMask(argv[4], &termtypes[0]); /* source */
+			DBTechNoisyNameMask(argv[5], &termtypes[1]); /* drain */
+			TTMaskAndMask3(&termtypes[2], &termtypes[0], &termtypes[1]);
+
+			if (TTMaskEqual(&termtypes[0], &termtypes[1]))
+			    termtypes[1] = DBSpaceBits;	/* Make is symmetric */
+			else if (!TTMaskIsZero(&termtypes[2]))
+			{
+			    TechError("Device mosfet %s has overlapping drain"
+				" and source types!\n", transName);
+			    /* Should this device be disabled? */
+			}
+			termtypes[2] = DBSpaceBits;
+			if (strcmp(argv[6], "None"))
+		    	DBTechNoisyNameMask(argv[6], &subsTypes);   /* substrate */
+			subsName = argv[7];
+			if (argc > 8) gscap = aToCap(argv[8]);
+			if (argc > 9) gccap = aToCap(argv[9]);
+			nterm = 2;
+		    }
+		    else
+		    {
+			/* Normal symmetric device with swappable source/drain */
+			
+			DBTechNoisyNameMask(argv[4], &termtypes[0]); /* source/drain */
+			termtypes[1] = DBSpaceBits;
+			if (strcmp(argv[5], "None"))
+			    DBTechNoisyNameMask(argv[5], &subsTypes);   /* substrate */
+			if (argc > 6) subsName = argv[6];
+			if (argc > 7) gscap = aToCap(argv[7]);
+			if (argc > 8) gccap = aToCap(argv[8]);
+			/* nterm = 1; */	/* Symmetric devices can be MOScaps */
+			nterm = 2;
+		    }
 		    break;
+
 		case DEV_DIODE:
-		    DBTechNoisyNameMask(argv[4], &types2);	/* negative types */
-		    subsTypes = DBZeroTypeBits;
+		    DBTechNoisyNameMask(argv[4], &termtypes[0]); /* negative types */
+		    termtypes[1] = DBSpaceBits;
 		    nterm = 1;
+		    if ((argc > 4) && strcmp(argv[4], "None"))
+			DBTechNoisyNameMask(argv[4], &subsTypes);   /* substrate */
+		    else
+			subsTypes = DBZeroTypeBits;
+		    if (argc > 5) subsName = argv[5];
 		    break;
+
 		case DEV_RES:
-		    DBTechNoisyNameMask(argv[4], &types2);	/* terminals */
-		    subsTypes = DBZeroTypeBits;
+		    DBTechNoisyNameMask(argv[4], &termtypes[0]); /* terminals */
+		    termtypes[1] = DBSpaceBits;
 		    nterm = 2;
+		    if ((argc > 5) && strcmp(argv[5], "None"))
+			DBTechNoisyNameMask(argv[5], &subsTypes);   /* substrate */
+		    else
+			subsTypes = DBZeroTypeBits;
+		    if (argc > 6) subsName = argv[6];
 		    break;
+
 		case DEV_CAP:
-		    DBTechNoisyNameMask(argv[4], &types2);	/* bottom */
-		    subsTypes = DBZeroTypeBits;
-		    if (argc > 5) gscap = aToCap(argv[5]);	/* perimeter cap */
+		    DBTechNoisyNameMask(argv[4], &termtypes[0]); /* bottom */
+		    termtypes[1] = DBSpaceBits;
+
 		    gccap = aToCap(argv[argc - 1]);		/* area cap */
+		    if ((argc > 6) && StrIsNumeric(argv[argc - 2]))
+		    {
+			gscap = aToCap(argv[argc - 2]);		/* perimeter cap */
+			argc--;
+		    }
 		    nterm = 1;
+
+		    if ((argc > 6) && strcmp(argv[5], "None"))
+			DBTechNoisyNameMask(argv[5], &subsTypes);   /* substrate */
+		    else
+			subsTypes = DBZeroTypeBits;
+		    if (argc > 7) subsName = argv[6];
 		    break;
+
 		case DEV_SUBCKT:
-		    nterm = 1;
-		    if (argc > 4)
-			DBTechNoisyNameMask(argv[4], &types2);	  /* terminals */
-		    if (argc > 5)
-			DBTechNoisyNameMask(argv[5], &subsTypes); /* substrate */
+		    /* Check final arguments for "x=y" statements showing what	*/
+		    /* parameter names the subcircuit uses.			*/
+
+		    while ((paramName = strchr(argv[argc - 1], '=')) != NULL)
+		    {
+			paramName++;
+			newParam = (ParamList *)mallocMagic(sizeof(ParamList));
+			newParam->pl_count = 0;
+			newParam->pl_param = *argv[argc - 1];
+			newParam->pl_name = StrDup((char **)NULL, paramName);
+			newParam->pl_next = subcktParams;
+			subcktParams = newParam;
+			argc--;
+		    }
+
+		    if (StrIsInt(argv[4]))
+		    {
+			nterm = atoi(argv[4]);
+			iterm = 4 + nterm;
+		    }
+		    else
+		    {
+			nterm = 1;
+			iterm = 4;
+		    }
+		    
+		    /* terminals */
+		    for (i = iterm; i < iterm + nterm; i++)
+			DBTechNoisyNameMask(argv[iterm], &termtypes[i - iterm]);
+		    termtypes[nterm] = DBSpaceBits;
+
+		    if (argc > i)
+			DBTechNoisyNameMask(argv[i], &subsTypes); /* substrate */
+		    if (argc > (i + 1)) subsName = argv[i + 1];
 		    break;
+
 		case DEV_RSUBCKT:
+		    /* Check final arguments for "x=y" statements showing what	*/
+		    /* parameter names the subcircuit uses.			*/
+
+		    while ((paramName = strchr(argv[argc - 1], '=')) != NULL)
+		    {
+			paramName++;
+			newParam = (ParamList *)mallocMagic(sizeof(ParamList));
+			newParam->pl_count = 0;
+			newParam->pl_param = *argv[argc - 1];
+			newParam->pl_name = StrDup((char **)NULL, paramName);
+			newParam->pl_next = subcktParams;
+			subcktParams = newParam;
+			argc--;
+		    }
+
+		    /* Now check number of parameters */
+		    if ((argc - 1) > dv->k_maxargs)
+		    {
+			while (subcktParams != NULL)
+			{
+			    freeMagic(subcktParams->pl_name);
+			    freeMagic(subcktParams);
+			    subcktParams = subcktParams->pl_next;
+			}
+			goto usage;
+		    }
+
 		    nterm = 2;
-		    DBTechNoisyNameMask(argv[4], &types2);	  /* terminals */
-		    subsTypes = DBZeroTypeBits;
+		    DBTechNoisyNameMask(argv[4], &termtypes[0]);	/* terminals */
+		    termtypes[1] = DBSpaceBits;
+
+		    if ((argc > 5) && strcmp(argv[5], "None"))
+			DBTechNoisyNameMask(argv[5], &subsTypes);   /* substrate */
+		    else
+			subsTypes = DBZeroTypeBits;
+		    if (argc > 6) subsName = argv[6];
 		    break;
 	    }
 
@@ -1890,7 +2059,15 @@ ExtTechLine(sectionName, argc, argv)
 		if (TTMaskHasType(&types1, t))
 		{
 		    TTMaskSetMask(ExtCurStyle->exts_transConn + t, &types1);
-		    ExtCurStyle->exts_transSDTypes[t] = types2;
+
+		    for (i = 0; !TTMaskHasType(&termtypes[i], TT_SPACE); i++);
+		    ExtCurStyle->exts_transSDTypes[t] = (TileTypeBitMask *)
+					mallocMagic((i + 1) * sizeof(TileTypeBitMask));
+			
+		    for (i = 0; !TTMaskHasType(&termtypes[i], TT_SPACE); i++)
+			ExtCurStyle->exts_transSDTypes[t][i] = termtypes[i];
+		    ExtCurStyle->exts_transSDTypes[t][i] = DBSpaceBits;
+
 		    ExtCurStyle->exts_transSDCount[t] = nterm;
 		    ExtCurStyle->exts_transSDCap[t] = gscap;
 		    ExtCurStyle->exts_transGateCap[t] = gccap;
@@ -1910,6 +2087,11 @@ ExtTechLine(sectionName, argc, argv)
 					exts_subsTransistorTypes[z], t);
 			}
 		    }
+		    if (subcktParams != NULL)
+		    {
+			ExtCurStyle->exts_deviceParams[t] = subcktParams;
+			subcktParams->pl_count++;
+		    }
 		}
 	    }
 	    break;
@@ -2171,7 +2353,9 @@ ExtTechLine(sectionName, argc, argv)
 	    }
 	    break;
 	case SIDEHALO:
-	    ExtCurStyle->exts_sideCoupleHalo = atoi(argv[1]);
+	    /* Allow floating-point and increase by factor of 1000	*/
+	    /* to accomodate "units microns".				*/
+	    ExtCurStyle->exts_sideCoupleHalo = (int)(1000 * (float)atof(argv[1]));
 	    break;
 	case PERIMC:
 	    DBTechNoisyNameMask(argv[2], &types2);
@@ -2186,23 +2370,29 @@ ExtTechLine(sectionName, argc, argv)
 			TTMaskSetType(&ExtCurStyle->exts_perimCapMask[s], t);
 		    }
 	    break;
-	case RESIST:
+	case RESIST: {
+	    float chop = 1.0;
+
 	    val = atoi(argv[2]);
+	    if (argc == 4)
+		chop = atof(argv[3]);
 	    class = ExtCurStyle->exts_numResistClasses++;
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 		if (TTMaskHasType(&types1, t))
 		{
 		    ExtCurStyle->exts_sheetResist[t] = val;
+		    ExtCurStyle->exts_cornerChop[t] = chop;
 		    ExtCurStyle->exts_typeToResistClass[t] = class;
 		}
 	    ExtCurStyle->exts_resistByResistClass[class] = val;
 	    ExtCurStyle->exts_typesByResistClass[class] = types1;
+	    }
 	    break;
 	case RSCALE:
 	    ExtCurStyle->exts_resistScale = atoi(argv[1]);
 	    break;
 	case STEP:
-	    val = atoi(argv[1]);
+	    val = (int)atof(argv[1]);
 	    if (val <= 0)
 	    {
 		TechError("Hierarchical interaction step size must be > 0\n");
@@ -2471,14 +2661,14 @@ zinit:
 
     if (doConvert)
     {
+	/* exts_unitsPerLambda is in centimicrons, so divide by		*/
+	/* 100 to get microns.						*/
+
+	CapValue scalefac = (CapValue)style->exts_unitsPerLambda / 100.0;
+	CapValue sqfac = scalefac * scalefac;
+
 	for (r = 0; r < DBNumTypes; r++)
 	{
-	    /* exts_unitsPerLambda is in centimicrons, so divide by	*/
-	    /* 100 to get microns, and invert to get the scalefactor.	*/
-
-	    CapValue scalefac = style->exts_unitsPerLambda / 100.0;
-	    CapValue sqfac = scalefac * scalefac;
-
 	    style->exts_areaCap[r] *= sqfac;
 	    style->exts_transSDCap[r] *= sqfac;
 	    style->exts_transGateCap[r] *= sqfac;
@@ -2489,12 +2679,36 @@ zinit:
 
 		style->exts_perimCap[r][s] *= scalefac;
 		style->exts_overlapCap[r][s] *= sqfac;
-		for (ec = style->exts_sideCoupleCap[r][s]; ec != NULL;
+		for (ec = style->exts_sideOverlapCap[r][s]; ec != NULL;
 				ec = ec->ec_next)
 		    ec->ec_cap *= sqfac;
+
+		for (ec = style->exts_sideCoupleCap[r][s]; ec != NULL;
+				ec = ec->ec_next)
+		    ec->ec_cap *= scalefac;
 	    }
 	}
+
+	/* side halo and step size are also in microns */
+
+	style->exts_sideCoupleHalo = (int)(((CapValue)style->exts_sideCoupleHalo
+		/ scalefac) + 0.5);
+	style->exts_stepSize = (int)(((CapValue)style->exts_stepSize
+		/ scalefac) + 0.5);
     }
+
+    /* Avoid setting stepSize to zero, or extraction will hang! */
+    if (style->exts_stepSize <= 0)
+    {
+	TxError("Warning:  zero step size!  Resetting to default.\n");
+	style->exts_stepSize = 100;		/* Revert to default */
+    }
+
+    /* We had multiplied sideCoupleHalo by 1000 to accomodate a 	*/
+    /* floating-point value in microns, whether or not doConvert was	*/
+    /* needed, so normalize it back to lambda units.			*/
+
+    style->exts_sideCoupleHalo /= 1000;
 }
 
 /*
@@ -2548,6 +2762,12 @@ ExtTechScale(scalen, scaled)
 	    style->exts_overlapCap[i][j] /= sqd;    /* Typo fixed in 7.2.57 */
 	    for (ec = style->exts_sideCoupleCap[i][j]; ec != NULL;
 				ec = ec->ec_next)
+	    {
+		ec->ec_cap *= scalen;
+		ec->ec_cap /= scaled;
+	    }
+	    for (ec = style->exts_sideOverlapCap[i][j]; ec != NULL;
+				ec = ec->ec_next)
 	    {
 		ec->ec_cap *= sqn;
 		ec->ec_cap /= sqd;
