@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtTest.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtTest.c,v 1.2 2008/06/01 18:37:42 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -485,7 +485,7 @@ extShowTrans(name, mask, out)
 	{
 	    fprintf(out, "    %-8.8s  %d terminals: ",
 			DBTypeShortName(t), ExtCurStyle->exts_transSDCount[t]);
-	    extShowMask(&ExtCurStyle->exts_transSDTypes[t], out);
+	    extShowMask(&ExtCurStyle->exts_transSDTypes[t][0], out);
 	    fprintf(out, "\n\tcap (gate-sd/gate-ch) = %lf/%lf\n",
 			ExtCurStyle->exts_transSDCap[t],
 			ExtCurStyle->exts_transGateCap[t]);
