@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtTimes.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtTimes.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -29,6 +29,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include <sys/param.h>
 #include <sys/times.h>
 #endif
+#include <math.h>		/* for sqrt() function */
 
 #include "utils/magic.h"
 #include "utils/utils.h"
@@ -908,7 +909,6 @@ extCumOutput(str, cum, f)
     struct cumStats *cum;
     FILE *f;
 {
-    double sqrt();
     double mean, var;
 
     mean = var = 0.0;
