@@ -3,7 +3,7 @@
  *
  * Defines the exported interface to the circuit extractor.
  *
- * rcsid "$Header: /usr/cvsroot/magic-8.0/extract/extract.h,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-8.0/extract/extract.h,v 1.2 2008/12/03 14:12:09 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -45,9 +45,8 @@ extern int ExtDoWarn;		/* Bitmask of above */
 #define DEV_RES         3		/* Resistor */
 #define DEV_CAP         4		/* Capacitor */
 #define DEV_DIODE	5		/* Diode */
-#define DEV_SUBCKT      6		/* FET-like (general-purpose)	*/
-					/* subcircuit.			*/
-#define DEV_RSUBCKT    7		/* Resistor-like subcircuit.	*/
+#define DEV_SUBCKT      6		/* general-purpose subcircuit	*/
+#define DEV_RSUBCKT     7		/* Resistor-like subcircuit.	*/
 
 /* Device names for .ext file output (new in version 7.2)	*/
 /* (defined in extract/ExtBasic.c *and* extflat/EFread.c)	*/
