@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResConDCS.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResConDCS.c,v 1.3 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -120,7 +120,7 @@ dbcConnectFuncDCS(tile, cx)
     {
          t2 = TiGetType(tp);
 	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
+	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
 	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
@@ -136,7 +136,7 @@ dbcConnectFuncDCS(tile, cx)
     {
          t2 = TiGetType(tp);
 	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
+	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
 	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
@@ -152,7 +152,7 @@ dbcConnectFuncDCS(tile, cx)
     {
          t2 = TiGetType(tp);
 	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
+	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
 	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
@@ -168,7 +168,7 @@ dbcConnectFuncDCS(tile, cx)
     {
          t2 = TiGetType(tp);
 	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
+	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
 	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
@@ -234,19 +234,25 @@ dbcConnectFuncDCS(tile, cx)
 
     if (DBIsContact(loctype))
     {
-	TileTypeBitMask *rMask = DBResidueMask(loctype);
+	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
 
 	TTMaskSetOnlyType(&notConnectMask, loctype);
 
 	/* Differenct contact types may share residues (6/18/04) */     
 	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
-	    if (TTMaskIntersect(rMask, DBResidueMask(ctype)))
+	{
+	    cMask = DBResidueMask(ctype);
+	    if (TTMaskIntersect(rMask, cMask))
 		TTMaskSetType(&notConnectMask, ctype);
+	}
 
 	/* The mask of contact types must include all stacked contacts */
 	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)   
-	    if TTMaskHasType(DBResidueMask(ctype), loctype)
+	{
+	    cMask = DBResidueMask(ctype);
+	    if TTMaskHasType(cMask, loctype)
 		TTMaskSetType(&notConnectMask, ctype);
+	}
 
 	TTMaskCom(&notConnectMask);
     }
@@ -463,7 +469,7 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
 	 for (tran = TT_TECHDEPBASE; tran < TT_MAXTYPES; tran++)
 	 {
 	      TTMaskSetMask(&DiffTypeBitMask,
-	      		&(ExtCurStyle->exts_transSDTypes[tran]));
+	      		&(ExtCurStyle->exts_transSDTypes[tran][0]));
 	      TTMaskSetMask(&ResSubsTypeBitMask,
 	      		&(ExtCurStyle->exts_transSubstrateTypes[tran]));
 	 }
