@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -69,7 +69,7 @@ ResInitializeConn()
      {
      	 for (diff = TT_TECHDEPBASE; diff < TT_MAXTYPES; diff++)
 	 {
-	     if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[tran]),diff)
+	     if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[tran][0]),diff)
 	     {
 		  TTMaskSetType(&ResConnectWithSD[diff],tran);
 	     }
@@ -854,7 +854,7 @@ FindStartTile(goodies, SourcePoint)
     for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp=RT(tp))
     {
 	t2 = TiGetRightType(tp);
-	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),t2))
+	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),t2))
 	{
 	    SourcePoint->p_x = LEFT(tile);
 	    SourcePoint->p_y = (MIN(TOP(tile),TOP(tp))+
@@ -867,7 +867,7 @@ FindStartTile(goodies, SourcePoint)
     for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp=LB(tp))
     {
 	t2 = TiGetLeftType(tp);
-	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),t2))
+	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),t2))
 	{
 	    SourcePoint->p_x = RIGHT(tile);
 	    SourcePoint->p_y = (MIN(TOP(tile),TOP(tp))+
@@ -880,7 +880,7 @@ FindStartTile(goodies, SourcePoint)
     for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp=BL(tp))
     {
 	t2 = TiGetBottomType(tp);
-	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),t2))
+	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),t2))
 	{
 	    SourcePoint->p_y = TOP(tile);
 	    SourcePoint->p_x = (MIN(RIGHT(tile),RIGHT(tp))+
@@ -893,7 +893,7 @@ FindStartTile(goodies, SourcePoint)
     for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp=TR(tp))
     {
 	t2 = TiGetTopType(tp);
-	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),t2))
+	if (TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),t2))
 	{
 	    SourcePoint->p_y = BOTTOM(tile);
 	    SourcePoint->p_x = (MIN(RIGHT(tile),RIGHT(tp))+
