@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResUtils.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResUtils.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -195,7 +195,7 @@ ResAddPlumbing(tile, arg)
 	       /* top */
 	       for (tp2= RT(tile); RIGHT(tp2) > LEFT(tile); tp2 = BL(tp2))
 	       {
-	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype]),
+	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype][0]),
 				TiGetBottomType(tp2))
 		    {
 			  junk2->sourceEdge |= TOPEDGE;
@@ -210,7 +210,7 @@ ResAddPlumbing(tile, arg)
 	       if (source == NULL)
 	       for (tp2= LB(tile); LEFT(tp2) < RIGHT(tile); tp2 = TR(tp2))
 	       {
-	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype]),
+	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype][0]),
 				TiGetTopType(tp2))
 		    {
 			  junk2->sourceEdge |= BOTTOMEDGE;
@@ -225,7 +225,7 @@ ResAddPlumbing(tile, arg)
 	       if (source == NULL)
 	       for (tp2= TR(tile); TOP(tp2) > BOTTOM(tile); tp2 = LB(tp2))
 	       {
-	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype]),
+	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype][0]),
 				TiGetLeftType(tp2))
 		    {
 			  junk2->sourceEdge |= RIGHTEDGE;
@@ -240,7 +240,7 @@ ResAddPlumbing(tile, arg)
 	       if (source == NULL)
 	       for (tp2= BL(tile); BOTTOM(tp2) < TOP(tile); tp2 = RT(tp2))
 	       {
-	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype]),
+	      	    if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[loctype][0]),
 				TiGetRightType(tp2))
 		    {
 			  source = tp2;
@@ -360,7 +360,7 @@ ResAddPlumbing(tile, arg)
 	       			    Junk->tj_status |= RES_TILE_TRAN;
 				    
 			       }
-	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),
+	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),
 				TiGetBottomType(tp2))
 			  {
 			       Junk = resAddField(tp2);
@@ -381,7 +381,7 @@ ResAddPlumbing(tile, arg)
 	       			    Junk->transistorList =  resFet;
 	       			    Junk->tj_status |= RES_TILE_TRAN;
 			       }
-	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),
+	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),
 				TiGetTopType(tp2))
 			  {
 			       Junk = resAddField(tp2);
@@ -402,7 +402,7 @@ ResAddPlumbing(tile, arg)
 	       			    Junk->transistorList =  resFet;
 	       			    Junk->tj_status |= RES_TILE_TRAN;
 			       }
-	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),
+	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),
 				TiGetLeftType(tp2))
 			  {
      	  		       Junk = resAddField(tp2);
@@ -423,7 +423,7 @@ ResAddPlumbing(tile, arg)
 	       			    Junk->transistorList =  resFet;
 	       			    Junk->tj_status |= RES_TILE_TRAN;
 			       }
-	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1]),
+	      	    	  else if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t1][0]),
 				TiGetRightType(tp2))
 			  {
      	  		       Junk = resAddField(tp2);
