@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/router/rtrTech.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/router/rtrTech.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -403,8 +403,8 @@ RtrTechFinal()
  */
 
 int
-RtrTechScale(scalen, scaled)
-    int scalen, scaled;
+RtrTechScale(scaled, scalen)
+    int scaled, scalen;
 {
     int i;
 
