@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selOps.c,v 1.2 2008/02/07 05:44:21 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selOps.c,v 1.9 2008/12/11 04:20:13 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -48,6 +48,13 @@ static int selStretchX, selStretchY;	/* Stretch distances.  Only one should
 					 */
 static TileType selStretchType;		/* Type of material being stretched. */
 
+typedef struct planeAndArea
+{
+    int pa_plane;			/* Plane of interest */
+    Rect *pa_area;			/* Area affected */
+    TileTypeBitMask *pa_mask;		/* Mask used in plane search */
+} planeAndArea;
+
 /* The following structure type is used to build up a list of areas
  * to be painted.  It's used to save information while a search of
  * the edit cell is in progress:  can't do the paints until the
@@ -119,7 +126,7 @@ SelectDelete(msg, do_clear)
     }
 
     DBReComputeBbox(EditCellUse->cu_def);
-    GeoTransRect(&RootToEditTransform, &SelectDef->cd_bbox, &editArea);
+    GeoTransRect(&RootToEditTransform, &SelectDef->cd_extended, &editArea);
     DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editArea);
@@ -237,6 +244,68 @@ SelectCopy(transform)
     SelectClear();
     SelectAndCopy2(EditRootDef);
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * SelectFlat --
+ *
+ * 	This procedure copies the selection into Select2Def, flattening
+ *	it as it copies, then copies the result back into the selection
+ *	cell.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The selection is flattened.  No changes are made in the edit cell.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+SelectFlat()
+{
+    SearchContext scx;
+
+    /* Copy from SelectDef to Select2Def while transforming, then
+     * let SelectAndCopy2 do the rest of the work.  Don't record
+     * anything involving Select2Def for undo-ing.
+     */
+
+    UndoDisable();
+    DBCellClearDef(Select2Def);
+    scx.scx_use = SelectUse;
+    scx.scx_area = SelectUse->cu_bbox;
+    GeoTransTrans(&GeoIdentityTransform, &SelectUse->cu_transform, &scx.scx_trans);
+    DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_ALL, Select2Use);
+    FlatCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_ALL, Select2Use);
+    DBReComputeBbox(Select2Def);
+    UndoEnable();
+
+    /* Move everything from Select2 to Select */
+
+    SelectClear();
+    SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
+
+    scx.scx_use = Select2Use;
+    scx.scx_area = Select2Use->cu_bbox;
+    GeoTransTrans(&GeoIdentityTransform, &Select2Use->cu_transform, &scx.scx_trans);
+
+    DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
+		SelectUse);
+    DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, SelectUse,
+		(Rect *)NULL);
+
+    SelRememberForUndo(FALSE, SelectRootDef, &SelectUse->cu_bbox);
+
+    /* Redisplay the select cell */
+    DBWHLRedraw(SelectRootDef, &SelectDef->cd_extended, TRUE);
+    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
+	&DBAllButSpaceBits);
+
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -369,16 +438,16 @@ selTransLabelFunc(label, cellUse, defTransform, transform)
     GeoTransRect(defTransform, &label->lab_rect, &rootArea);
     rootJust = GeoTransPos(defTransform, label->lab_just);
     GeoTransPointDelta(defTransform, &label->lab_offset, &rootOffset);
-    rootRotate = GeoTransAngle(transform, label->lab_rotate);
+    rootRotate = GeoTransAngle(defTransform, label->lab_rotate);
 
     GeoTransRect(transform, &rootArea, &finalArea);
     finalJust = GeoTransPos(transform, rootJust);
     GeoTransPointDelta(transform, &rootOffset, &finalOffset);
     finalRotate = GeoTransAngle(transform, rootRotate);
 
-    (void) DBPutFontLabel(Select2Def, &finalArea, label->lab_size, 
-	    finalRotate, &finalOffset, finalJust, label->lab_text,
-	    label->lab_type, label->lab_flags);
+    (void) DBPutFontLabel(Select2Def, &finalArea, label->lab_font,
+	    label->lab_size, finalRotate, &finalOffset, finalJust,
+	    label->lab_text, label->lab_type, label->lab_flags);
     return 0;
 }
 
@@ -742,8 +811,8 @@ SelectStretch(x, y)
      * displayer and DRC about it later.
      */
 
-    modifiedArea = Select2Def->cd_bbox;
-    (void) GeoInclude(&SelectDef->cd_bbox, &modifiedArea);
+    modifiedArea = Select2Def->cd_extended;
+    (void) GeoInclude(&SelectDef->cd_extended, &modifiedArea);
     GeoTransRect(&RootToEditTransform, &modifiedArea, &editModified);
 
     /* Delete the selection itself. */
@@ -833,8 +902,12 @@ selStretchEraseFunc(tile, plane)
 {
     Rect area, editArea;
     int planeNum;
+    planeAndArea pa;
     TileType type, t;
-    TileTypeBitMask *typeResidueMask, *connectMask, tmpmask, mask;
+    TileTypeBitMask tmpmask, mask, *residueMask;
+    PaintUndoInfo ui;
+    PaintResultType selStretchEraseTbl[NT];
+    extern int selStretchEraseFunc2();
 
 #ifdef NONMANHATTAN
     if (IsSplit(tile))
@@ -864,75 +937,33 @@ selStretchEraseFunc(tile, plane)
 
     GeoTransRect(&RootToEditTransform, &area, &editArea);
  
-    /* Was previously DBEraseMask(), and needs to be again to avoid	*/
-    /* quirky behavior.  However, the DBErase() call is worse.  We need	*/
-    /* to erase all types that interact with "type", *not* all types on	*/
-    /* "plane", due to the way stacked contacts are handled.  But	*/
-    /* contacts on different planes may stretch across one another	*/
-    /* without effect, if the types are stackable.  So, getting the	*/
-    /* correct mask is not so simple.					*/
-
-    /* DBEraseMask(EditCellUse->cu_def, &editArea, &DBPlaneTypes[*plane]); */
-    /* DBErase(EditCellUse->cu_def, &editArea, type); */
-
-    /* Routine modified by Nishit; code below written 9/28/2004, added	*/
-    /* 10/14/04								*/
-
-    /*
-     *   Process mask first
-     */
+    /* We need to erase all types that interact with "type", *not* all	*/
+    /* types on "plane", due to the way stacked contacts are handled.	*/
+    /* Contacts on different planes may stretch across one another	*/
+    /* without effect, if the types are stackable.  Likewise, layer	*/
+    /* types may stretch across contacts for which they are a residue.	*/
 
     planeNum = *plane;
-    typeResidueMask = DBResidueMask(type);
     mask = DBPlaneTypes[planeNum];
-    connectMask = &DBConnectTbl[type];
-    for (t = TT_SPACE + 1; t < DBNumTypes; t++)
+    if (DBIsContact(type))
     {
-	/* Process all tile types that exist on plane
-	 * "planeNum" (except the current one, as we want
-	 *  to erase that type of tile from the layout).
-	 */
-	if (TTMaskHasType(&mask, t) && (t != type))
+	for (t = DBNumUserLayers; t < DBNumTypes; t++)
 	{
-	    if (DBIsContact(t))
-	    {
-	        /* While stretching non-contact material
-	         * we would not process residue type of "t".
-	         * Because that may remove mask of "type" from
-	         * "mask" which would untouch that material in
-	         * stretching path.
-	         */
-		if (DBIsContact(type))
-	        {
-		     tmpmask = *DBResidueMask(t);
-	             if (!TTMaskIsZero(typeResidueMask) &&
-	                	!TTMaskIsZero(&tmpmask))
-                        TTMaskAndMask3(&tmpmask, typeResidueMask, &tmpmask);
+	    if (t == type) continue;
 
-	             /* If t is stacked type then we would not
-	              * remove its residue mask from "mask".
-	              */
-	             if (!TTMaskIsZero(&tmpmask) && (t < DBNumUserLayers))
-	                 TTMaskClearMask(&mask, &tmpmask);
-	         }
-	
-                 /* If tile type "t" connects to tile "type"
-                  * remove that type from mask. We would not
-                  * like to erase that type of tiles.
-                  */
-                 if (TTMaskHasType(connectMask, t))
-                 {
-                     /* In case "type" is a stacked type and both "t" and
-		      * "type" are on same plane, erase that tile (e.g.,
-		      * stack of 3 types selected)
-                      */
-                     if ((type < DBNumUserLayers) &&
-                        	(DBPlane(type) != DBPlane(t)))
-                          TTMaskClearType(&mask, t);
-                 }
-             }
-         }
-      }
+	    /* Exclude all contact types for which this type should "pass through" */
+
+	    if (TTMaskHasType(&mask, t))
+	    {
+		tmpmask = *DBResidueMask(t);
+		if (TTMaskHasType(&tmpmask, type))
+		{
+		    TTMaskClearType(&tmpmask, type);
+		    TTMaskClearMask(&mask, &tmpmask);
+		}
+	    }
+	}
+    }
 
     /* For stacked type tiles do not erase common residue
      * type of both contacts from layout.
@@ -946,41 +977,73 @@ selStretchEraseFunc(tile, plane)
 
     TTMaskAndMask(&mask, &DBActiveLayerBits);
 
-    /*
-     *   Erase all tile types specified in the processed mask.
+    /* Erase all tile types specified in the processed mask.
+     * Erase contacts in mask first using DBErase.  This prevents
+     * contacts from leaving partial images of themselves on other
+     * planes.  Then, remove the non-contact layers on this plane
+     * only, such that contacts are not disturbed.
      */
+
+    TTMaskZero(&tmpmask);
     	
-    /* New DBEraseMask() routine for stretching. */
-    /* TODO : Placed here just for checking funtionality.
-     * Once works properly shift to DBtcontact.c.
-     */
+    selStretchEraseTbl[TT_SPACE] = (PaintResultType)TT_SPACE;
+    for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
     {
-       /* Erase contacts first from mask . Because if you
-        * erase one type of tile from contact first then you
-        * can't remove that contact completely as its residue
-        * type remains on other plane. And that will stay for ever !
-        */
-
-       if (DBIsContact(type))
-       {
-           for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
-	       if (TTMaskHasType(&mask, t) && DBIsContact(t))
-		   if (!LockedLayer(t))
-			DBErase(EditCellUse->cu_def, &editArea, t);
-       }
-       for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
-           if (TTMaskHasType(&mask, t) && !DBIsContact(t))
-		if (!LockedLayer(t))
+	selStretchEraseTbl[t] = (PaintResultType)t;
+	if (TTMaskHasType(&mask, t))
+	{
+	    if (DBIsContact(t))
+	    {
+		if (t == type)
 		    DBErase(EditCellUse->cu_def, &editArea, t);
+		else
+		    TTMaskSetType(&tmpmask, t);
+	    }
+	    else
+		selStretchEraseTbl[t] = (PaintResultType)TT_SPACE;
+	}
     }
+    for (; t < DBNumTypes; t++)
+	selStretchEraseTbl[t] = (PaintResultType)t;
 
-    /*
-     *   Completed.
-     */
+    /* Remove conflicting contact types, where they exist */
+
+    pa.pa_area = &editArea;
+    pa.pa_plane = planeNum;
+    pa.pa_mask = &tmpmask;
+    DBSrPaintArea((Tile *)NULL, EditCellUse->cu_def->cd_planes[planeNum],
+		&editArea, &tmpmask, selStretchEraseFunc2, (ClientData)&pa);
+
+    ui.pu_pNum = planeNum;
+    ui.pu_def = EditCellUse->cu_def;
+    DBPaintPlane(EditCellUse->cu_def->cd_planes[planeNum], &editArea,
+		selStretchEraseTbl, &ui);
 
     return 0;
 }
-
+
+int
+selStretchEraseFunc2(tile, pa)
+    Tile *tile;
+    planeAndArea *pa;
+{
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+    {
+	if (TTMaskHasType(pa->pa_mask, TiGetLeftType(tile)))
+	    DBErase(EditCellUse->cu_def, pa->pa_area,
+			DBPlaneToResidue(TiGetLeftType(tile), pa->pa_plane));
+	if (TTMaskHasType(pa->pa_mask, TiGetRightType(tile)))
+	    DBErase(EditCellUse->cu_def, pa->pa_area,
+			DBPlaneToResidue(TiGetRightType(tile), pa->pa_plane));
+    }
+    else
+#endif
+	DBErase(EditCellUse->cu_def, pa->pa_area,
+		DBPlaneToResidue(TiGetType(tile), pa->pa_plane));
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *	selStretchFillFunc --
@@ -1129,7 +1192,7 @@ selStretchFillFunc3(tile, area)
 {
     Rect editArea, rootArea;
     TileType type, stype;
-    TileTypeBitMask *mask;
+    TileTypeBitMask *mask, *sMask, *tMask;
     StretchArea *sa;
 
     /* Compute the area to be painted. */
@@ -1181,6 +1244,7 @@ selStretchFillFunc3(tile, area)
     else
 #endif
     type = TiGetType(tile);
+    sMask = DBResidueMask(selStretchType);
 
     /* If have type and stretch type are both contacts (fixed types)	*/
     /* and there exists a stacking contact type between them, then	*/
@@ -1194,7 +1258,7 @@ selStretchFillFunc3(tile, area)
 	/* fill with type.  Otherwise, we check the list of plow	*/
 	/* types to see what is and isn't fixed.			*/
 
-	if (!TTMaskHasType(DBResidueMask(selStretchType), type))
+	if (!TTMaskHasType(sMask, type))
 	    if (TTMaskHasType(&PlowFixedTypes, type)
 			|| !TTMaskHasType(&PlowFixedTypes, selStretchType))
 		type = selStretchType;
@@ -1215,8 +1279,8 @@ selStretchFillFunc3(tile, area)
 	    ((selStretchY > 0) && (editArea.r_ybot == area->r_ytop)))
 	{
 	    TileTypeBitMask rmask;
-	    TTMaskAndMask3(&rmask, DBResidueMask(type),
-			DBResidueMask(selStretchType));
+	    tMask = DBResidueMask(type);
+	    TTMaskAndMask3(&rmask, tMask, sMask);
 	    for (type = TT_TECHDEPBASE; type < DBNumUserLayers; type++)
 		if (TTMaskHasType(&rmask, type))
 		    break;
