@@ -23,11 +23,7 @@ static char sccsid[] = "@(#)SimSelect.c	4.14 MAGIC (Berkeley) 10/3/85";
 #endif  /* not lint */
 
 #include <stdio.h>
-#ifdef SYSV
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -280,7 +276,7 @@ SimSelectArea(Rect *rect)
 	}
 
 	HashKill(&SimAbortSeenTbl);
-	ExtResetTiles(SelectDef, (ClientData) MINFINITY);
+	ExtResetTiles(SelectDef, (ClientData) CLIENTDEFAULT);
 	SimGetNodeCleanUp();
 	SimRecomputeSel = FALSE;
     }
@@ -824,9 +820,9 @@ SimEraseLabels()
     for (p = SimCellLabList; p; p = p->dl_next) {
 	if (p->dl_isMarked) {
 	    p->dl_isMarked = FALSE;
-	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, -1, "RSIM=X");
-	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, -1, "RSIM=1");
-	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, -1, "RSIM=0");
+	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, "RSIM=X");
+	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, "RSIM=1");
+	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, "RSIM=0");
 	}
     }
 }
