@@ -9,14 +9,14 @@
 # revision C: Adds a layer manager toolbar on the left side
 # revision D: Adds a menubar on top with cell and tech manager tools
 
-global treeViews
+global MagicBLT
 global windowsopen
 global Glyph
 global Opts
 global Winopts
 
-set treeViews 0
-if {![catch {package require BLT}]} {set treeViews 1}
+set MagicBLT 0
+if {! [catch {package require BLT}]} {set MagicBLT 1}
 
 # Simple console commands (like TkCon, but much simpler)
 
@@ -92,6 +92,27 @@ proc magic::promptload {type} {
 		magic::load [file root $Layoutfilename]
 	    }
 	  }
+
+      getcell { set Layoutfilename [ tk_getOpenFile -filetypes \
+		{{Magic {.mag {.mag}}} {"All files" {*}}}]
+	    if {$Layoutfilename != ""} {
+		set fdir [file dirname $Layoutfilename]
+		set lidx [lsearch [path search] $fdir]
+		if {$lidx < 0} {path search +$fdir}
+		magic::getcell [file tail $Layoutfilename]
+
+		# Append path to cell search path if it's not there already
+
+		if {[string index $Layoutfilename 0] != "/"} {
+		   set $Layoutfilename "./$Layoutfilename"
+		}
+		set sidx [string last "/" $Layoutfilename]
+		if {$sidx > 0} {
+		   set cellpath [string range $Layoutfilename 0 $sidx]
+		   magic::path cell +$cellpath
+		}
+	    }
+	  }
    }
 }
 
@@ -232,17 +253,17 @@ menu .winmenu -tearoff 0
 # The cell manager
 
 proc magic::makecellmanager { mgrpath } {
-   global treeViews
+   global MagicBLT
 
-   if {$treeViews} {
+   if {$MagicBLT} {
       toplevel ${mgrpath}
       wm withdraw ${mgrpath}
       frame ${mgrpath}.actionbar
       frame ${mgrpath}.box
       frame ${mgrpath}.target
-      blt::hiertable ${mgrpath}.box.view -bg white -allowduplicates false \
-		-hideroot true -yscrollcommand "${mgrpath}.box.vert set" \
-		-xscrollcommand "${mgrpath}.box.vert set"
+      blt::hiertable ${mgrpath}.box.view -bg white -allowduplicates false -hideroot true \
+	   -yscrollcommand "${mgrpath}.box.vert set" \
+	   -xscrollcommand "${mgrpath}.box.vert set"
       scrollbar ${mgrpath}.box.vert -orient vertical -command "${mgrpath}.box.view yview"
 
       pack ${mgrpath}.actionbar -side top -fill x
@@ -273,6 +294,18 @@ proc magic::makecellmanager { mgrpath } {
    }
 }
 
+
+proc magic::setgrid {gridsize} {
+   set techlambda [magic::tech lambda]
+   set tech1 [lindex $techlambda 1]
+   set tech0 [lindex $techlambda 0]
+   set tscale [expr {$tech1 / $tech0}]
+   set lambdaout [expr {[magic::cif lambda output] * $tscale}]
+   set gridlambda [expr {$gridsize/$lambdaout}]
+   magic::grid ${gridlambda}l
+   magic::snap on
+}
+
 proc magic::addlistinstance {instin} {
    set hierinst [join [lrange $instin 1 end] "/"]
    set defname [magic::instance list celldef $hierinst]
@@ -287,8 +320,8 @@ proc magic::addlistinstance {instin} {
 }
 
 proc magic::cellmanager {{option "update"}} {
-   global treeViews
-   if {$treeViews} {
+   global MagicBLT
+   if {$MagicBLT} {
 
       # Check for existence of the manager widget
       if {[catch {wm state .cellmgr}]} {
@@ -323,6 +356,134 @@ proc magic::cellmanager {{option "update"}} {
    }
 }
 
+# Text helper window
+
+proc magic::maketexthelper { mgrpath } {
+   toplevel ${mgrpath}
+   wm withdraw ${mgrpath}
+
+   frame ${mgrpath}.title
+   frame ${mgrpath}.text
+   frame ${mgrpath}.font
+   frame ${mgrpath}.size
+   frame ${mgrpath}.just
+   frame ${mgrpath}.rotate
+   frame ${mgrpath}.offset
+
+   frame ${mgrpath}.buttonbar
+
+   label ${mgrpath}.title.tlab -text "Selected labels: "
+
+   label ${mgrpath}.text.tlab -text "Text string: "
+   label ${mgrpath}.font.tlab -text "Font: "
+   label ${mgrpath}.size.tlab -text "Size: "
+   label ${mgrpath}.just.tlab -text "Justification: "
+   label ${mgrpath}.rotate.tlab -text "Rotation: "
+   label ${mgrpath}.offset.tlab -text "Offset from reference: "
+
+   entry ${mgrpath}.text.tent -background white
+   entry ${mgrpath}.size.tent -background white
+   entry ${mgrpath}.rotate.tent -background white
+   entry ${mgrpath}.offset.tent -background white
+
+   menubutton ${mgrpath}.just.btn -text "default" -menu ${mgrpath}.just.btn.menu
+   menubutton ${mgrpath}.font.btn -text "default" -menu ${mgrpath}.font.btn.menu
+   
+   button ${mgrpath}.buttonbar.cancel -text "Cancel" -command "wm withdraw ${mgrpath}"
+   button ${mgrpath}.buttonbar.apply -text "Apply"
+
+   pack ${mgrpath}.title.tlab
+   pack ${mgrpath}.font.tlab -side left
+   pack ${mgrpath}.font.btn -side left
+   pack ${mgrpath}.text.tlab -side left
+   pack ${mgrpath}.text.tent -side right -fill x -expand true
+   pack ${mgrpath}.size.tlab -side left
+   pack ${mgrpath}.size.tent -side right -fill x -expand true
+   pack ${mgrpath}.just.tlab -side left
+   pack ${mgrpath}.just.btn -side left
+   pack ${mgrpath}.rotate.tlab -side left
+   pack ${mgrpath}.rotate.tent -side right -fill x -expand true
+   pack ${mgrpath}.offset.tlab -side left
+   pack ${mgrpath}.offset.tent -side right -fill x -expand true
+   pack ${mgrpath}.buttonbar.apply -side left
+   pack ${mgrpath}.buttonbar.cancel -side right
+
+   pack ${mgrpath}.title -side top
+   pack ${mgrpath}.font -side top -anchor w
+   pack ${mgrpath}.text -side top -anchor w -expand true
+   pack ${mgrpath}.size -side top -anchor w -expand true
+   pack ${mgrpath}.just -side top -anchor w
+   pack ${mgrpath}.rotate -side top -anchor w -expand true
+   pack ${mgrpath}.offset -side top -anchor w -expand true
+   pack ${mgrpath}.buttonbar -side bottom -fill x -expand true
+}
+
+# Create or redisplay the text helper
+
+proc magic::texthelper {{option "update"}} {
+   global CAD_ROOT
+
+   if {[catch {wm state .texthelper}]} {
+      if {$option == "create"} {
+	 magic::maketexthelper .texthelper
+      } else {
+	 return
+      }
+   } elseif { $option == "create"} {
+      return
+   }
+
+   if {$option == "create"} {
+      menu .texthelper.just.btn.menu -tearoff 0
+      menu .texthelper.font.btn.menu -tearoff 0
+
+      .texthelper.just.btn.menu add command -label "NW" -command \
+		"magic::setlabel just nw ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "N" -command \
+		"magic::setlabel just n ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "NE" -command \
+		"magic::setlabel just ne ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "E" -command \
+		"magic::setlabel just e ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "SE" -command \
+		"magic::setlabel just se ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "S" -command \
+		"magic::setlabel just s ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "SW" -command \
+		"magic::setlabel just sw ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "W" -command \
+		"magic::setlabel just w ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "NW" -command \
+		"magic::setlabel just nw ; \
+		 magic::texthelper update"
+      .texthelper.just.btn.menu add command -label "center" -command \
+		"magic::setlabel just c ; \
+		 magic::texthelper update"
+   }
+
+   if {$option == "init"} {
+      .texthelper.font.btn.menu delete 0 end
+   }
+
+   if {$option ==  "init" || $option == "create"} {
+      set flist [magic::setlabel fontlist]
+      foreach i $flist {
+         .texthelper.font.btn.menu add command -label $i -command \
+		"magic::setlabel font $i ; \
+		 magic::texthelper update"
+      }
+   }
+}
+
+
 # Technology manager callback functions
 
 proc magic::techparseunits {} {
@@ -529,7 +690,6 @@ proc magic::captions {{subcommand {}}} {
    }
    set winlist [magic::windownames layout]
    foreach winpath $winlist {
-
       set framename [winfo parent $winpath]
       set caption [$winpath windowcaption]
       set subcaption1 [lindex $caption 0]
@@ -568,7 +728,7 @@ magic::tag see "magic::toolupdate %W %1 %2"
 magic::tag tech "magic::techrebuild %W %1; magic::captions %1"
 magic::tag drc "magic::drcupdate %1"
 magic::tag path "magic::techmanager update"
-if {$treeViews} {
+if {$MagicBLT} {
    magic::tag cellname "magic::mgrupdate %W %1"
    magic::tag select "magic::mgrselect %R"
    magic::tag cif "magic::mgrupdate %W %1"
@@ -734,7 +894,7 @@ proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
 
 # Redirect and reformat Tcl output of "select" command
 
-if {$treeViews} {
+if {$MagicBLT} {
    proc magic::mgrselect {{sstr ""}} {
       # Make sure we have a valid option, and the cell manager exists.
       if {$sstr == ""} {
@@ -777,22 +937,20 @@ if {$treeViews} {
       }
       magic::tag select $savetag
    }
-}
 
-# Update cell and tech managers in response to a cif or gds read command
+   # Update cell and tech managers in response to a cif or gds read command
 
-proc magic::mgrupdate {win {cmdstr ""}} {
-   global treeViews
-
-   if {${cmdstr} == "read"} {
-      if {$treeViews} { magic::cellmanager }
-      magic::captions
-      magic::techmanager update
-   } elseif {${cmdstr} == "delete" || ${cmdstr} == "rename"} {
-      if {$treeViews} { magic::cellmanager }
-      magic::captions
-   } elseif {${cmdstr} == "writeable"} {
-      magic::captions
+   proc magic::mgrupdate {win {cmdstr ""}} {
+      if {${cmdstr} == "read"} {
+	 magic::cellmanager
+	 magic::captions
+	 magic::techmanager update
+      } elseif {${cmdstr} == "delete" || ${cmdstr} == "rename"} {
+	 magic::cellmanager
+	 magic::captions
+      } elseif {${cmdstr} == "writeable"} {
+	 magic::captions
+      }
    }
 }
 
@@ -1069,7 +1227,7 @@ proc magic::makescrollbar { fname orient win } {
 
 proc magic::openwrapper {{cell ""} {framename ""}} {
    global windowsopen
-   global treeViews
+   global MagicBLT
    global Glyph
    global Opts
    global Winopts
@@ -1115,9 +1273,15 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    menubutton ${framename}.titlebar.mbuttons.file -text File -relief raised \
 		-menu ${framename}.titlebar.mbuttons.file.filemenu -borderwidth 2
+   menubutton ${framename}.titlebar.mbuttons.edit -text Edit -relief raised \
+		-menu ${framename}.titlebar.mbuttons.edit.editmenu -borderwidth 2
+   menubutton ${framename}.titlebar.mbuttons.view -text View -relief raised \
+		-menu ${framename}.titlebar.mbuttons.view.viewmenu -borderwidth 2
    menubutton ${framename}.titlebar.mbuttons.opts -text Options -relief raised \
 		-menu ${framename}.titlebar.mbuttons.opts.optsmenu -borderwidth 2
    pack ${framename}.titlebar.mbuttons.file -side left
+   pack ${framename}.titlebar.mbuttons.edit -side left
+   pack ${framename}.titlebar.mbuttons.view -side left
    pack ${framename}.titlebar.mbuttons.opts -side left
 
    # DRC status button
@@ -1180,6 +1344,8 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		[${winname} cellname list window]"
    $m add command -label "Close" -command "magic::closewrapper ${framename}"
    $m add separator
+   $m add command -label "Place Cell" -command {magic::promptload getcell}
+   $m add separator
    $m add command -label "Load layout" -command {magic::promptload magic}
    $m add command -label "Read CIF" -command {magic::promptload cif}
    $m add command -label "Read GDS" -command {magic::promptload gds}
@@ -1194,6 +1360,49 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    $m add separator
    $m add command -label "Quit" -command {magic::quit}
 
+   # Generate the edit menu
+
+   set m [menu ${framename}.titlebar.mbuttons.edit.editmenu -tearoff 0]
+   $m add command -label "Undo          (u)" -command {magic::undo}
+   $m add command -label "Redo          (U)" -command {magic::redo}
+   $m add separator
+   $m add command -label "Rotate 90 degree " -command {magic::clock}
+   $m add command -label "Flip Horizontal" -command {magic::sideways}
+   $m add command -label "Flip Vertical"   -command {magic::upsidedown}
+   $m add separator
+   $m add command -label "Text ..." \
+		-command [subst { magic::texthelper create; \
+		wm deiconify .texthelper ; raise .texthelper } ]
+   $m add separator
+   $m add command -label "Lock   Cell      " -command {magic::instance lock}
+   $m add command -label "Unlock Cell      " -command {magic::instance unlock}
+   $m add separator
+   $m add command -label "Unlock Base Layers" -command {magic::tech layer unlock *}
+
+   # Generate the view menu
+
+   set m [menu ${framename}.titlebar.mbuttons.view.viewmenu -tearoff 0]
+   $m add command -label "Zoom In   " -command {magic::zoom 0.5}
+   $m add command -label "Zoom Out      (Z)" -command {magic::zoom 2}
+   $m add command -label "Zoom Sel      (z)" -command {magic::findbox zoom}
+   $m add command -label "Full" -command {magic::select top cell ; findbox zoom}
+   $m add separator
+   $m add command -label "Grid 0.10u        " -command {magic::setgrid 0.1}
+   $m add command -label "Grid 0.20u        " -command {magic::setgrid 0.2}
+   $m add command -label "Grid 0.25u        " -command {magic::setgrid 0.25}
+   $m add command -label "Grid 0.40u        " -command {magic::setgrid 0.4}
+   $m add command -label "Grid 0.5u        " -command {magic::setgrid 0.5}
+   $m add command -label "Grid 1.0u        " -command {magic::setgrid 1}
+   $m add command -label "Grid 2.0u        " -command {magic::setgrid 2}
+   $m add command -label "Grid 4.0u        " -command {magic::setgrid 4}
+   $m add command -label "Grid 5.0u        " -command {magic::setgrid 5}
+   $m add command -label "Grid 10.0u        " -command {magic::setgrid 10}
+   $m add command -label "Grid 50.0u        " -command {magic::setgrid 50}
+   $m add separator
+   $m add command -label "Expand Toggle (/)" -command {magic::expand toggle}
+   $m add command -label "Expand        (x)" -command {magic::expand }
+   $m add command -label "Unexpand      (X)" -command {magic::unexpand }
+
    # Generate the options menu
 
    set m [menu ${framename}.titlebar.mbuttons.opts.optsmenu -tearoff 0]
@@ -1205,7 +1414,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		grid forget ${framename}.toolbar } }]
 
    .winmenu add radio -label ${framename} -variable Opts(target) -value ${winname}
-   if {$treeViews} {
+   if {$MagicBLT} {
       $m add check -label "Cell Manager" -variable Opts(cellmgr) \
 		-command [subst { magic::cellmanager create; \
 		if { \$Opts(cellmgr) } { \
@@ -1216,7 +1425,6 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		-text ${framename}"
    }
 
-
    $m add check -label "Tech Manager" -variable Opts(techmgr) \
 	-command [subst { magic::techmanager create; \
 		if { \$Opts(techmgr) } { \
@@ -1295,21 +1503,22 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 # Delete the wrapper and the layout window in it.
 
 proc magic::closewrapper { framename } {
-   global treeViews
+   global MagicBLT
    global Opts
 
-   # Remove this window from the target list in cellmanager
+   # Remove this window from the target list in .winmenu
+   # (used by, e.g., cellmanager)
 
    if { $Opts(target) == "${framename}.magic" } {
       set Opts(target) "default"
-      if {$treeViews} {
-         if {![catch {wm state .cellmgr}]} {
+      if {$MagicBLT} {
+	 if {![catch {wm state .cellmgr}]} {
 	    .cellmgr.target.list configure -text "default"
-	 }
+         } 
       }
    }
 
-   set idx [.winmenu index $framename ]
+   set idx [.winmenu index $framename]
    .winmenu delete $idx
 
    ${framename}.magic magic::closewindow
