@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/utils/hash.c,v 1.2 2001/01/12 22:13:31 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/hash.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -241,7 +241,7 @@ hash(table, key)
 
 	/* Just use the 32-bit key value */
 	case HT_WORDKEYS:
-	    i = (int) key;
+	    i = (spointertype) key;
 	    break;
 
 	/* Special case for two-word structs */
