@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/macros/macros.c,v 1.6 2001/09/26 18:24:39 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/macros.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -156,9 +156,66 @@ MacroRetrieve(client, xc, iReturn)
 	    }
 	}
     }
+    if (iReturn != NULL) *iReturn = FALSE;
     return (char *)NULL;
 }
 
+/*---------------------------------------------------------
+ * MacroSubsitute:
+ *	Make a substitution of one string for another
+ *	in a macro string.  This is really just a string
+ *	manipulation;  it doesn't have anything specifically
+ *	to do with macros.
+ *
+ * Results:
+ *	A new string pointer to the macro contents.
+ *
+ * Side Effects:
+ *	None.
+ *---------------------------------------------------------
+ */
+
+char *
+MacroSubstitute(macrostr, searchstr, replacestr)
+    char *macrostr;
+    char *searchstr;
+    char *replacestr;
+{
+    char *found, *last, *new;
+    int expand, length, oldlength, srchsize;
+
+    if (macrostr == (char *)NULL) return NULL;
+
+    oldlength = strlen(macrostr);
+    srchsize = strlen(searchstr);
+    expand = strlen(replacestr) - srchsize;
+    last = macrostr;
+    length = oldlength;
+    while ((found = strstr(last, searchstr)) != NULL)
+    {
+	length += expand;
+	last = found + srchsize;
+    }
+    if (length > oldlength)
+    {
+	new = (char *)mallocMagic(length + 1);
+	*new = '\0';
+	last = macrostr;
+	while ((found = strstr(last, searchstr)) != NULL)
+	{
+	   *found = '\0';
+	   strcat(new, last);
+	   strcat(new, replacestr);
+	   last = found + srchsize;
+	}
+	strcat(new, last);
+	freeMagic(macrostr);
+	return new;
+    }
+    else
+	return macrostr;
+}
+
 
 /*---------------------------------------------------------
  * MacroDelete:
@@ -359,10 +416,11 @@ MacroKey(str, verbose)
 	}
 	if (!strncmp(vis, "XK_", 3)) vis += 3;
 
-	/* We're converting all ASCII back into X Keycodes 	   */
-	/* The original macro (.magic) file format allows embedded */
-	/* control characters, so we have to handle those.  For    */
-	/* regular ASCII characters, the keysym = the ASCII value. */
+	/* We're converting all ASCII back into X Keycodes 	*/
+	/* The original macro (.magicrc) file format allows	*/
+	/* embedded control characters, so we have to handle	*/
+	/* those.  For regular ASCII characters, the keysym =	*/
+	/* the ASCII value.					*/
 
 	if ((*(vis + 1)) == '\0')
 	{
