@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/utils/niceabort.c,v 1.2 2005/05/11 17:39:32 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/niceabort.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -57,7 +57,7 @@ char AbortMessage[200];
 bool AbortFatal = FALSE;
 
 /* For lint */
-long time();
+time_t time();
 char *ctime();
 
 void
