@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/path.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/path.c,v 1.2 2008/06/01 18:37:47 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -185,8 +185,10 @@ PaExpand(psource, pdest, size)
 	size -= length;
 	pd = *pdest+length;
     }
-    else if (*ps == '$')	/* Environment variable expansion */
+    else if (*ps == '$')	/* (Possible) environment variable expansion */
     {
+	char saveChar;
+
 	pd = expandName;
 	for (i=0; ; i++)
 	{
@@ -195,6 +197,7 @@ PaExpand(psource, pdest, size)
 		break;
 	    if (i < 99) pd++;
 	}
+	saveChar = *pd;
 	*pd = '\0';
 
 	/* Lookup the environment variable in the hash table.  Make a new
@@ -228,9 +231,9 @@ PaExpand(psource, pdest, size)
 		string = CAD_DIR;
 	    else
 	    {
-		while ((*ps != '\0') && !isspace(*ps) && (*ps != ':')) ps++;
-		*psource = ps;
-		return -1;
+		*pd = saveChar;
+		ps = *psource;
+		goto noexpand;
 	    }
 	}
 
@@ -251,6 +254,7 @@ PaExpand(psource, pdest, size)
 	 * then just skip over them.
 	 */
 
+noexpand:
 	while (ps[0] == '.')
 	{
 	    if (ps[1] == '/') ps += 2;
