@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/utils/runstats.c,v 1.2 2001/01/12 22:13:33 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/runstats.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
@@ -131,7 +131,7 @@ RunStats(flags, lastt, deltat)
 
     if (flags & RS_MEM)
     {
-	size = (((int) sbrk(0) - (int) &end) + 512)/1024;
+	size = (((long) sbrk(0) - (int) &end) + 512)/1024;
 	if (sp != string)
 	    *sp++ = ' ';
 	sprintf(sp, "%dk", size);
