@@ -16,7 +16,7 @@
  * Magic utility routines.
  */
 
-/* rcsid "$Header: /ufs/repository/magic/utils/utils.h,v 1.5 2001/08/23 13:48:58 tim Exp $" */
+/* rcsid "$Header: /usr/cvsroot/magic-8.0/utils/utils.h,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $" */
 
 #ifndef _UTILS_H
 #define _UTILS_H 1
@@ -44,6 +44,7 @@ extern int LookupStructFull();
 extern int PaExpand();
 extern char *nextName();
 extern FILE *PaOpen();
+extern FILE *PaLockOpen();
 extern char *StrDup();
 extern int Match();
 extern char *ArgStr();
@@ -51,16 +52,10 @@ extern bool StrIsWhite();
 extern bool StrIsInt();
 extern bool StrIsNumeric();
 
-#ifdef FILE_LOCKS
-#if defined(_STDIO_H) || defined(_STDIO_H_)
-extern FILE *flock_open();
-extern FILE *flock_test_set();
-#endif
-extern int flock_close();
-extern void flock_print();
-extern int flock_exit();
+extern int SetNoisyBool(bool *, char *, FILE *);
 
-extern int flock_flag;
+#ifdef FILE_LOCKS
+extern FILE *flock_open();
 #endif
 
 /* The following macro takes an integer and returns another integer that
